from django.contrib import admin
from .models import Post, Category, Tags


class CategoryAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug', 'description', 'is_active')


class PostAdmin(admin.ModelAdmin):
    list_display = ('title', 'slug', 'user', 'category', 'status')


class TagsAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug')


admin.site.register(Category, CategoryAdmin)
admin.site.register(Post, PostAdmin)
admin.site.register(Tags, TagsAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-03-30 05:39
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_blog_it', '0004_posthistory'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('role', models.CharField(choices=[('Drafted', 'Drafted'), ('Published', 'Published'), ('Rejected', 'Rejected'), ('Trashed', 'Trashed')], max_length=10)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterField(
            model_name='image_file',
            name='thumbnail',
            field=models.FileField(blank=True, null=True, upload_to='static/uploads/%Y/%m/%d/'),
        ),
        migrations.AlterField(
            model_name='image_file',
            name='upload',
            field=models.FileField(upload_to='static/uploads/%Y/%m/%d/'),
        ),
        migrations.AlterField(
            model_name='post',
            name='status',
            field=models.CharField(choices=[('Drafted', 'Drafted'), ('Published', 'Published'), ('Rejected', 'Rejected'), ('Trashed', 'Trashed')], default='Drafted', max_length=10),
        ),
    ]



from django import template

register = template.Library()


class RecurseNode(template.Node):
    def __init__(self, var, name, child, nodeList):
        self.var = var
        self.name = name
        self.child = child
        self.nodeList = nodeList

    def __repr__(self):
        return '<RecurseNode>'

    def renderCallback(self, context, vals, level):
        output = []
        try:
            if len(vals):
                pass
        except:
            vals = [vals]
        if len(vals):
            if 'loop' in self.nodeList:
                output.append(self.nodeList['loop'].render(context))
            for val in vals:
                context.push()
                context['level'] = level
                context[self.name] = val
                if 'child' in self.nodeList:
                    output.append(self.nodeList['child'].render(context))
                    child = self.child.resolve(context)
                    if child:
                        output.append(self.renderCallback(context, child, level + 1))
                if 'endloop' in self.nodeList:
                    output.append(self.nodeList['endloop'].render(context))
                else:
                    output.append(self.nodeList['endrecurse'].render(context))
                context.pop()
            if 'endloop' in self.nodeList:
                output.append(self.nodeList['endrecurse'].render(context))
        return ''.join(output)

    def render(self, context):
        vals = self.var.resolve(context)
        output = self.renderCallback(context, vals, 1)
        return output


def do_recurse(parser, token):
    bits = list(token.split_contents())
    if len(bits) != 6 and bits[2] != 'with' and bits[4] != 'as':
        raise template.TemplateSyntaxError("Invalid tag syxtax expected '{% recurse [childVar] with [parents] as [parent] %}'")
    child = parser.compile_filter(bits[1])
    var = parser.compile_filter(bits[3])
    name = bits[5]

    nodeList = {}
    while len(nodeList) < 4:
        temp = parser.parse(('child', 'loop', 'endloop', 'endrecurse'))
        tag = parser.tokens[0].contents
        nodeList[tag] = temp
        parser.delete_first_token()
        if tag == 'endrecurse':
            break

    return RecurseNode(var, name, child, nodeList)
do_recurse = register.tag('recurse', do_recurse)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import datetime


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=20)),
                ('slug', models.CharField(unique=True, max_length=20)),
                ('description', models.CharField(max_length=500)),
                ('is_active', models.BooleanField(default=False)),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)),
            ],
        ),
        migrations.CreateModel(
            name='Image_File',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('upload', models.FileField(upload_to=b'static/uploads/%Y/%m/%d/')),
                ('date_created', models.DateTimeField(default=datetime.datetime.now)),
                ('is_image', models.BooleanField(default=True)),
                ('thumbnail', models.FileField(null=True, upload_to=b'static/uploads/%Y/%m/%d/', blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=100)),
                ('slug', models.SlugField(max_length=100)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('updated_on', models.DateField(auto_now=True)),
                ('content', models.TextField()),
                ('tags', models.TextField(null=True, blank=True)),
                ('status', models.CharField(default=b'Drafted', max_length=10, choices=[(b'Drafted', b'Drafted'), (b'Published', b'Published'), (b'Rejected', b'Rejected')])),
                ('keywords', models.TextField(max_length=500, blank=True)),
                ('category', models.ForeignKey(to='django_blog_it.Category', on_delete=models.CASCADE)),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)),
            ],
        ),
        migrations.CreateModel(
            name='Tags',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=20)),
                ('slug', models.CharField(unique=True, max_length=20)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-04-05 06:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_blog_it', '0005_auto_20160330_0539'),
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100)),
                ('content', models.TextField()),
                ('slug', models.SlugField()),
                ('is_active', models.BooleanField(default=True)),
                ('meta_description', models.TextField()),
                ('keywords', models.TextField()),
                ('meta_title', models.TextField()),
            ],
        ),
    ]



from django.test import TestCase
from django.test import Client
from django_blog_it.django_blog_it.models import Category, Post, Tags, PostHistory, UserRole, Page
from django.contrib.auth.models import User
from django_blog_it.django_blog_it.forms import BlogCategoryForm, BlogPostForm, AdminLoginForm
from django.core.urlresolvers import reverse
from django_blog_it.django_blog_it.models import Menu, Theme
from .forms import UserForm


# models test
class category_models_test(TestCase):

    def create_category(self, name="simple page", description="simple page content"):
        user = User.objects.create_superuser('mp@mp.com', 'micro-test', 'mp')
        return Category.objects.create(name=name, description=description, user=user)

    def test_category_creation(self):
        w = self.create_category()
        self.assertTrue(isinstance(w, Category))
        self.assertEqual(w.__str__(), w.name)


# models test
class tags_models_test(TestCase):

    def create_tags(self, name="simple page"):
        return Tags.objects.create(name=name)

    def test_category_creation(self):
        w = self.create_tags()
        self.assertTrue(isinstance(w, Tags))
        self.assertEqual(w.__str__(), w.name)


# models test
class pages_models_test(TestCase):

    def create_pages(self, title="simple page", content="simple content", meta_description="meta description", meta_title="meta title"):
        return Page.objects.create(title=title, content=content, meta_description=meta_description, meta_title=meta_title)

    def test_page_creation(self):
        w = self.create_pages()
        self.assertTrue(isinstance(w, Page))
        self.assertEqual(w.__str__(), w.title)


# models test
class post_models_test(TestCase):

    def create_post(
            self,
            tag="simple page",
            category="simple page",
            description="simple page content",
            title="post",
            content="content",
            status="D"
            ):
        user = User.objects.create_superuser('mp@mp.com', 'micro-test', 'mp')
        category = Category.objects.create(name=category, description=description, user=user)
        tag = Tags.objects.create(name=tag)
        return Post.objects.create(category=category, user=user, content=content, title=title, status=status)

    def test_category_creation(self):
        w = self.create_post()
        self.assertTrue(isinstance(w, Post))
        self.assertEqual(w.__str__(), w.title)


class post_history_models_test(TestCase):

    def create_post_history(self, content="simple content"):
        user = User.objects.create_superuser('mp@mp.com', 'micro-test', 'mp')
        category = Category.objects.create(name='category', description='description', user=user)
        post = Post.objects.create(category=category, user=user, content='content', title='title', status='Published')

        return PostHistory.objects.create(content=content, post=post, user=user)

    def test_category_creation(self):
        w = self.create_post_history()
        self.assertTrue(isinstance(w, PostHistory))
        self.assertEqual(w.__str__(), str(w.user.get_username()) + ' ' + str(w.content) + ' ' + str(w.post.title))


# class image_file_models_test(TestCase):

#     def create_image_file(self, content="simple content"):
#         upload_file = open('/django_blog_it/static/favicon.png', 'rb')
#         return Image_File.objects.create(Image_File=upload_file, thumbnail=upload_file, upload=upload_file)

#     def test_category_creation(self):
#         w = self.create_image_file()
#         self.assertTrue(isinstance(w, Image_File))
#         self.assertEqual(w.__str__(), str(w.date_created()))


class django_blog_it_forms_test(TestCase):

    def get_category(self):
        self.category2 = Category.objects.create(name='generators',
                                                 description='generators',
                                                 user=self.user)
        return self.category2

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(
            'mp@mp.com', 'mp', 'mp')
        self.employee = User.objects.create_user(
            'mp@micropyramid.com', 'mp', 'mp')
        self.category = Category.objects.create(
            name='salesforce', description='salesforce desc', user=self.user, is_active=True)
        self.blogppost = Post.objects.create(
            title='python introduction',
            user=self.user,
            content='This is content',
            category=self.category,
            status='Published')

    def test_blogpostform(self):
        data = {'title': 'jquery introduction',
                'content': 'This is content',
                'category': self.category.id,
                'status': 'Published',
                'is_superuser': 'True',
                'slug': 'jquery-introduction'}
        form = BlogPostForm(data=data)
        self.assertTrue(form.is_valid())
        form = BlogPostForm(data=data, user_role='Author')
        self.assertTrue(form.is_valid())

    def test_BlogCategoryForm(self):
        data = {'name': 'django form',
                'description': 'django',
                'user': self.category.id}
        form = BlogCategoryForm(data=data)
        self.assertTrue(form.is_valid())
        form = BlogCategoryForm(data=data)
        self.assertTrue(form.is_valid())
        data['name'] = 'salesforce'
        form = BlogCategoryForm(data=data)
        self.assertFalse(form.is_valid())
        data['name'] = self.category.name
        form = BlogCategoryForm(data=data, instance=self.get_category())
        self.assertFalse(form.is_valid())

    def test_AdminLoginForm(self):
        form = AdminLoginForm(
            data={'username': 'mp@mp.com', 'password': 'mp'})
        self.assertTrue(form.is_valid())

        form = AdminLoginForm(
            data={'username': 'mp@mp.com', 'password': 'mp123'})
        self.assertFalse(form.is_valid())

        form = AdminLoginForm(
            data={'username': 'mp@micropyramid.com', 'password': 'mp'})
        self.assertTrue(form.is_valid())

    def tearDown(self):
        super(django_blog_it_forms_test, self).tearDown()
        if hasattr(self, 'category2'):
            self.category2.delete()


class django_blog_it_views_get(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(
            'mp@mp.com', 'micro-test', 'mp')
        self.employee = User.objects.create_user(
            'mp@micropyramid.com', 'micro', 'mp')
        self.category = Category.objects.create(
            name='django', description='django desc', user=self.user, is_active=True)
        self.linuxcategory = Category.objects.create(
            name='linux', description='django desc', user=self.user, is_active=True)
        self.blogppost = Post.objects.create(
            title='other python introduction',
            user=self.user,
            content='This is content',
            category=self.category,
            status='Published',
            slug="other-python-introduction")
        self.tag = Tags.objects.create(name='testtag')
        self.blogppost.tags.add(self.tag)
        self.pythonpost = Post.objects.create(
            title='decorator',
            user=self.user,
            content='This is content',
            category=self.category,
            status='Published',
            slug="decorator")

    def test_blog_get(self):

        response = self.client.get('/dashboard/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/new_admin-login.html')

        response = self.client.post(
            '/dashboard/', {'email': 'mp@mp.com', 'password': 'micro-test'})
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            '/dashboard/', {'email': 'mp@mp.com', 'password': 'micro'})
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            '/dashboard/', {'email': 'mp@micropyramid.com', 'password': 'micro'})
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            '/dashboard/', {'email': 'mp@micropyramid.com', 'password': 'micro-test'})
        self.assertEqual(response.status_code, 200)

        response = self.client.get('/dashboard/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/new_admin-login.html')

        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)

        response = self.client.get('/dashboard/view/' + str(self.blogppost.slug) + '/')
        self.assertEqual(response.status_code, 200)
        # self.assertTemplateUsed(response, 'dashboard/blog/blog_view.html')

        response = self.client.get('/dashboard/logout/')
        self.assertEqual(response.status_code, 302)

    def test_blog_post(self):

        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)

        response = self.client.get('/dashboard/')
        self.assertEqual(response.status_code, 302)

        response = self.client.get('/dashboard/blog/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/blog/new_blog_list.html')

        response = self.client.post('/dashboard/blog/', {'select_status': '', 'search_text': ''})
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/blog/new_blog_list.html')

        response = self.client.post('/dashboard/blog/', {'select_status': 'Published', 'search_text': ''})
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/blog/new_blog_list.html')

        response = self.client.post(
            '/dashboard/blog/',
            {
                'select_status': 'Published',
                'search_text': str(self.category.id)
            })
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/blog/new_blog_list.html')

        response = self.client.post('/dashboard/blog/', {'select_status': '', 'search_text': str(self.category.id)})
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/blog/new_blog_list.html')

        response = self.client.get('/dashboard/category/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/category/new_categories_list.html')

        response = self.client.post('/dashboard/category/', {'select_status': '', 'category': []})
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/category/new_categories_list.html')

        response = self.client.post('/dashboard/category/', {'select_status': 'True', 'category': []})
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/category/new_categories_list.html')

        response = self.client.post('/dashboard/category/', {'select_status': 'False', 'category': []})
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/category/new_categories_list.html')

        response = self.client.post(
            '/dashboard/category/',
            {
                'select_status': 'Published',
                'category': [str(self.category.id)]
            })
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/category/new_categories_list.html')

        response = self.client.post('/dashboard/category/', {'select_status': '', 'category': [str(self.category.id)]})
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/category/new_categories_list.html')

        response = self.client.get('/dashboard/category/add/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/category/new_category_add.html')

        response = self.client.post(
            '/dashboard/category/add/',
            {
                'name': 'python',
                'description': 'Python description',
                'user': str(self.user.id)
            })
        self.assertEqual(response.status_code, 200)
        self.assertTrue('Successfully added your category' in str(response.content))

        response = self.client.post(
            '/dashboard/category/add/', {'description': 'python'})
        self.assertEqual(response.status_code, 200)
        self.assertFalse('Successfully added your category' in str(response.content))

        response = self.client.get('/dashboard/category/edit/django/')
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            '/dashboard/category/edit/django/', {'name': 'django', 'description': 'django', 'user': str(self.user.id)})
        self.assertEqual(response.status_code, 200)
        self.assertTrue('Successfully updated your category' in str(response.content))

        response = self.client.post(
            '/dashboard/category/edit/django/', {'name': 'jquery', 'description': 'django', 'user': str(self.user.id)})
        self.assertEqual(response.status_code, 200)
        self.assertTrue('Successfully updated your category' in str(response.content))

        response = self.client.post(
            '/dashboard/category/edit/python/', {'description': 'python'})
        self.assertEqual(response.status_code, 200)
        self.assertFalse('Successfully updated your category' in str(response.content))

    def test_blog_with_super_admin(self):

        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)

        response = self.client.get('/dashboard/view/' + str(self.blogppost.slug) + '/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/blog/new_blog_view.html')

        response = self.client.get('/dashboard/upload_photos/', {'CKEditorFuncNum': '/dashboard/'})
        self.assertEqual(response.status_code, 200)
        context = {'CKEditorFuncNum': '/dashboard/'}
        response = self.client.get('/dashboard/upload_photos/', context)
        self.assertEqual(response.status_code, 200)

        # recent photos
        response = self.client.get('/dashboard/recent_photos/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/browse.html')

        response = self.client.get('/dashboard/bulk_actions_blog/')
        self.assertEqual(response.status_code, 200)

        response = self.client.get('/dashboard/bulk_actions_blog/', {'blog_ids[]': [str(self.blogppost.id)]})
        self.assertEqual(response.status_code, 200)

        response = self.client.get(
            '/dashboard/bulk_actions_blog/',
            {
                'blog_ids[]': [str(self.blogppost.id)],
                'action': 'Published'
            })
        self.assertEqual(response.status_code, 200)

        response = self.client.get(
            '/dashboard/bulk_actions_blog/',
            {
                'blog_ids[]': [str(self.pythonpost.id)],
                'action': 'Delete'
            })
        self.assertEqual(response.status_code, 200)

        # bulk actions category
        response = self.client.get('/dashboard/bulk_actions_category/')
        self.assertEqual(response.status_code, 200)

        response = self.client.get('/dashboard/bulk_actions_category/', {'blog_ids[]': [str(self.category.id)]})
        self.assertEqual(response.status_code, 200)

        response = self.client.get(
            '/dashboard/bulk_actions_category/',
            {
                'blog_ids[]': [str(self.category.id)],
                'action': 'True'
            })
        self.assertEqual(response.status_code, 200)

        response = self.client.get(
            '/dashboard/bulk_actions_category/',
            {
                'blog_ids[]': [str(self.linuxcategory.id)],
                'action': 'Delete'
            })
        self.assertEqual(response.status_code, 200)

        response = self.client.get(
            '/dashboard/bulk_actions_category/',
            {
                'blog_ids[]': [str(self.category.id)],
                'action': 'False'
            })
        self.assertEqual(response.status_code, 200)

        # delete category
        response = self.client.post(
            '/dashboard/category/add/',
            {
                'name': 'python',
                'description': 'Python description',
                'user': str(self.user.id)
            })
        response = self.client.get('/dashboard/category/delete/python/')
        self.assertEqual(response.status_code, 302)


class blog_post_creation(TestCase):

    def get_author_role(self):
        self.author_role = UserRole.objects.create(user=self.user, role='Author')

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(
            'mp@mp.com', 'mp', 'mp')
        self.employee = User.objects.create_user(
            'mp@micropyramid.com', 'mp', 'mp')
        self.category = Category.objects.create(
            name='salesforce', description='salesforce desc', user=self.user, is_active=True)
        self.post = Post.objects.create(title="apache", slug="apache", category=self.category, user=self.user)

    def test_blog_post_add(self):
        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)

        response = self.client.get('/dashboard/add/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/blog/new_blog_add.html')

        response = self.client.post(
            reverse("blog_add"),
            {
                'title': 'python introduction',
                'content': 'This is content',
                'category': self.category.id,
                'status': 'Published',
                'tags': 'django',
                'is_superuser': 'True',
                'slug': 'python-introduction-1',
                'slugs-MAX_NUM_FORMS': ['1000'],
                'slugs-TOTAL_FORMS': ['3'], 'slugs-MIN_NUM_FORMS': ['0'],
                'slugs-0-slug': ['python-introduction-1'], 'slugs-1-slug': [''],
                'slugs-2-slug': [''], 'slugs-INITIAL_FORMS': ['0'],
            })
        self.assertEqual(response.status_code, 200)
        # self.assertTrue('Successfully posted your blog' in str(response.content))

        response = self.client.post(
            '/dashboard/add/',
            {
                'title': 'python introduction',
                'content': '',
                'category': self.category.id,
                'status': 'Published',
                'tags': 'django',
                'is_superuser': 'True',
                'slug': 'python-introduction-1'
            })
        self.assertEqual(response.status_code, 200)
        self.assertFalse('Successfully posted your blog' in str(response.content))

        response = self.client.post('/dashboard/add/', {'content': '', 'title': ''})
        self.assertEqual(response.status_code, 200)
        self.assertFalse('Successfully posted your blog' in str(response.content))

        response = self.client.post(
            '/dashboard/add/',
            {
                'title': 'testing',
                'content': 'This is content',
                'category': self.category.id,
                'status': 'Published',
                'tags': 'django',
                'is_superuser': 'True',
                'slug': 'testing',
                'slugs-MAX_NUM_FORMS': ['1000'],
                'slugs-TOTAL_FORMS': ['3'], 'slugs-MIN_NUM_FORMS': ['0'],
                'slugs-0-slug': ['testing-11223'], 'slugs-1-slug': [''],
                'slugs-2-slug': [''], 'slugs-INITIAL_FORMS': ['0'],
            })
        self.assertEqual(response.status_code, 200)
        # self.assertTrue('Successfully posted your blog' in str(response.content))

    def test_blog_post_edit(self):
        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)

        response = self.client.post(
            '/dashboard/add/',
            {
                'title': 'nginx post',
                'content': 'This is content',
                'category': self.category.id,
                'status': 'Published',
                'tags': 'django',
                'is_superuser': 'True',
                'slug': 'nginx-post',
                'slugs-MAX_NUM_FORMS': ['1000'],
                'slugs-TOTAL_FORMS': ['3'], 'slugs-MIN_NUM_FORMS': ['0'],
                'slugs-0-slug': ['testing-11223'], 'slugs-1-slug': [''],
                'slugs-2-slug': [''], 'slugs-INITIAL_FORMS': ['0'],
            })
        self.assertEqual(response.status_code, 200)
        # self.assertTrue('Successfully posted your blog' in str(response.content))
        response = self.client.get(reverse("edit_blog", kwargs={"blog_slug": self.post.slug}))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/blog/new_blog_add.html')

        response = self.client.post(
            reverse('edit_category', kwargs={"category_slug": self.category.slug}),
            {
                'title': 'nginx-post',
                'content': 'This is content',
                'category': self.category.id,
                'status': 'Published',
                'tags': 'django',
                'is_superuser': 'True',
                'slug': 'nginx-post',
                'slugs-MAX_NUM_FORMS': ['1000'],
                'slugs-TOTAL_FORMS': ['3'], 'slugs-MIN_NUM_FORMS': ['0'],
                'slugs-0-slug': ['nginx-post-1'], 'slugs-1-slug': [''],
                'slugs-2-slug': [''], 'slugs-INITIAL_FORMS': ['0'],
            })
        self.assertEqual(response.status_code, 200)
        # self.assertTrue('Successfully updated your blog post' in str(response.content))

        response = self.client.post(
            reverse("edit_blog", kwargs={"blog_slug": self.post.slug}),
            {
                'title': 'nginx-post',
                'content': '',
                'category': self.category.id,
                'status': 'Published',
                'tags': 'django',
                'is_superuser': 'True',
                'slug': 'nginx-post-1'
            })
        self.assertEqual(response.status_code, 200)
        self.assertFalse('Successfully updated your blog post' in str(response.content))

        response = self.client.post(reverse("edit_blog", kwargs={"blog_slug": self.post.slug}), {'content': '', 'title': ''})
        self.assertEqual(response.status_code, 200)
        self.assertFalse('Successfully updated your blog post' in str(response.content))

        response = self.client.post(
            reverse("edit_blog", kwargs={"blog_slug": self.post.slug}),
            {
                'title': 'nginx-post',
                'content': 'This is content',
                'category': self.category.id,
                'status': 'Published',
                'tags': 'django',
                'is_superuser': 'True',
                'slug': 'nginx-post-1',
                'slugs-MAX_NUM_FORMS': ['1000'],
                'slugs-TOTAL_FORMS': ['4'], 'slugs-MIN_NUM_FORMS': ['0'],
                'slugs-0-slug': ['python-introduction-1'],
                'slugs-1-slug': [''], 'slugs-2-slug': [''],
                'slugs-3-slug': [''], 'slugs-0-id': ['2'],
                'slugs-INITIAL_FORMS': ['1'], 'slugs-0-is_active': ['on'],
                'slugs-1-id': [''], 'slugs-2-id': [''], 'slugs-3-id': [''],
            })
        self.assertEqual(response.status_code, 200)
        # self.assertTrue('Successfully updated your blog post' in str(response.content))
        self.post = Post.objects.first()
        response = self.client.post(
            reverse("edit_blog", kwargs={"blog_slug": self.post.slug}),
            {
                'title': 'nginx-post',
                'content': 'This is content',
                'category': self.category.id,
                'status': 'Published',
                'tags': 'nginx',
                'is_superuser': 'True',
                'slug': 'nginx-post-1',
                'slugs-MAX_NUM_FORMS': ['1000'],
                'slugs-TOTAL_FORMS': ['4'], 'slugs-MIN_NUM_FORMS': ['0'],
                'slugs-0-slug': ['python-introduction-1'],
                'slugs-1-slug': [''], 'slugs-2-slug': [''],
                'slugs-3-slug': [''], 'slugs-0-id': ['2'],
                'slugs-INITIAL_FORMS': ['1'], 'slugs-0-is_active': ['on'],
                'slugs-1-id': [''], 'slugs-2-id': [''], 'slugs-3-id': [''],
            })
        self.assertEqual(response.status_code, 200)
        # self.assertTrue('Successfully updated your blog post' in str(response.content))

        self.user.is_superuser = False
        self.user.save()
        self.get_author_role()
        response = self.client.post(
            reverse("edit_blog", kwargs={"blog_slug": self.post.slug}),
            {
                'title': 'nginx-post',
                'content': 'This is content',
                'category': self.category.id,
                'status': 'Published',
                'tags': 'nginx',
                'is_superuser': 'True',
                'slug': 'nginx-post-1',
                'slugs-MAX_NUM_FORMS': ['1000'],
                'slugs-TOTAL_FORMS': ['4'], 'slugs-MIN_NUM_FORMS': ['0'],
                'slugs-0-slug': ['python-introduction-1'],
                'slugs-1-slug': [''], 'slugs-2-slug': [''],
                'slugs-3-slug': [''], 'slugs-0-id': ['2'],
                'slugs-INITIAL_FORMS': ['1'], 'slugs-0-is_active': ['on'],
                'slugs-1-id': [''], 'slugs-2-id': [''], 'slugs-3-id': [''],
            })
        self.assertEqual(response.status_code, 200)

    def test_blog_post_delete(self):
        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)

        response = self.client.post(
            '/dashboard/add/',
            {
                'title': 'haystack post',
                'content': 'This is content',
                'category': self.category.id,
                'status': 'Published',
                'tags': 'django',
                'is_superuser': 'True',
                'slug': 'haystack-post',
                'slugs-MAX_NUM_FORMS': ['1000'],
                'slugs-TOTAL_FORMS': ['3'], 'slugs-MIN_NUM_FORMS': ['0'],
                'slugs-0-slug': ['haystack-post-1'], 'slugs-1-slug': [''],
                'slugs-2-slug': [''], 'slugs-INITIAL_FORMS': ['0'],
            })
        self.assertEqual(response.status_code, 200)
        # self.assertTrue('Successfully posted your blog' in str(response.content))

        response = self.client.get(reverse("delete_category", kwargs={"category_slug": self.category.slug}))
        self.assertEqual(response.status_code, 302)
        self.post = Post.objects.create(title="apache2", slug="apache2", category=self.category, user=self.user)
        response = self.client.post(reverse("delete_blog", kwargs={"blog_slug": self.post.slug}), {'action': 'trash'})
        self.assertEqual(response.status_code, 302)

        response = self.client.post(reverse("delete_blog", kwargs={"blog_slug": self.post.slug}), {'action': 'restore'})
        self.assertEqual(response.status_code, 302)

        response = self.client.post(reverse("delete_blog", kwargs={"blog_slug": self.post.slug}), {'action': 'save'})
        self.assertEqual(response.status_code, 404)

        response = self.client.post(reverse("delete_blog", kwargs={"blog_slug": self.post.slug}), {'action': 'delete'})
        self.assertEqual(response.status_code, 302)

    def tearDown(self):
        super(blog_post_creation, self).tearDown()
        if hasattr(self, 'author_role'):
            self.author_role.delete()


class users_roles(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(
            'mp@mp.com', 'mp', 'mp')
        self.user_role = UserRole.objects.create(user=self.user, role='Admin')
        self.employee = User.objects.create_user(
            'mp@micropyramid.com', 'mp', 'mp')
        UserRole.objects.create(user=self.employee, role='Admin')
        # self.employee_role = UserRole.objects.create(user=self.employee, role='Author')

    def test_users_list(self):
        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)

        response = self.client.get('/dashboard/users/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/user/new_list.html')

        response = self.client.get('/dashboard/users/', {'select_role': ''})
        self.assertEqual(response.status_code, 200)

        response = self.client.get('/dashboard/users/', {'select_role': 'Admin'})
        self.assertEqual(response.status_code, 200)

        response = self.client.get('/dashboard/users/', {'select_role': 'Author'})
        self.assertEqual(response.status_code, 200)
        response = self.client.get('/dashboard/users/', {'select_role': 'Admin'})
        self.assertEqual(response.status_code, 200)

    def test_users_edit_delete(self):
        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)

        response = self.client.get('/dashboard/user/edit/' + str(self.user.id) + '/')
        self.assertEqual(response.status_code, 200)
        # self.assertTemplateUsed(response, 'dashboard/user/new_user_role.html')

        response = self.client.post('/dashboard/user/edit/' + str(self.user.id) + '/', {'role': 'Publisher'})
        self.assertEqual(response.status_code, 200)
        # self.assertTrue('Successfully Updated User Role' in str(response.content))

        response = self.client.post('/dashboard/user/edit/' + str(self.employee.id) + '/', {'role': 'Author'})
        self.assertEqual(response.status_code, 200)
        # self.assertTrue('Successfully Updated User Role' in str(response.content))

        response = self.client.post('/dashboard/user/edit/' + str(self.employee.id) + '/', {'role': ''})
        self.assertEqual(response.status_code, 200)
        self.assertFalse('Successfully Updated User Role' in str(response.content))

        response = self.client.get('/dashboard/user/delete/' + str(self.employee.id) + '/')
        self.assertEqual(response.status_code, 302)

        response = self.client.get('/dashboard/user/delete/' + str(self.employee.id+1) + '/')
        self.assertEqual(response.status_code, 404)


class Pages(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(
            'mp@mp.com', 'mp', 'mp')
        self.user_role = UserRole.objects.create(user=self.user, role='Admin')
        self.employee = User.objects.create_user(
            'mp@micropyramid.com', 'mp', 'mp')
        self.page = Page.objects.create(
            title="test", content="test content", meta_description='page desc', keywords='keywords', meta_title="meta title")

    def test_pages_list(self):
        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)

        response = self.client.get(reverse('pages'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/pages/new_list.html')

        response = self.client.get(reverse('pages'), {'select_status': 'True'})
        self.assertEqual(response.status_code, 200)

        response = self.client.get(reverse('pages'), {'select_status': 'False'})
        self.assertEqual(response.status_code, 200)

        response = self.client.get(reverse('bulk_actions_pages'), {'page_ids[]': [str(self.page.id)]})
        self.assertEqual(response.status_code, 200)

        response = self.client.get(reverse('bulk_actions_pages'), {'page_ids[]': [str(self.page.id)], 'action': 'True'})
        self.assertEqual(response.status_code, 200)

        response = self.client.get(reverse('bulk_actions_pages'), {'page_ids[]': [str(self.page.id)], 'action': 'False'})
        self.assertEqual(response.status_code, 200)

        response = self.client.get(reverse('bulk_actions_pages'), {'page_ids[]': [str(self.page.id)], 'action': 'Delete'})
        self.assertEqual(response.status_code, 200)
        response = self.client.get(reverse('bulk_actions_pages'), {'action': 'Delete'})
        self.assertEqual(response.status_code, 200)

    def test_pages_add(self):
        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)

        response = self.client.post(
            reverse('add_page'),
            {
                'title': 'nginx post',
                'content': 'This is content',
                'meta_description': 'page meta data',
                'meta_title': 'meta title',
                'keywords': 'django',
            })
        self.assertEqual(response.status_code, 200)
        self.assertTrue('Successfully added your page' in str(response.content))

        response = self.client.post(
            reverse('add_page'),
            {
                'title': '',
                'content': 'This is content',
                'meta_description': 'page meta data',
                'meta_title': 'meta title',
                'keywords': 'django',
            })
        self.assertEqual(response.status_code, 200)
        self.assertFalse('Successfully added your page' in str(response.content))

        response = self.client.get(reverse('add_page'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/pages/new_add_page.html')

        response = self.client.get(reverse('edit_page', kwargs={'page_slug': self.page.slug}))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'dashboard/pages/new_add_page.html')

        response = self.client.post(
            reverse('edit_page', kwargs={'page_slug': self.page.slug}),
            {
                'title': '',
                'content': 'This is content',
                'meta_description': 'page meta data',
                'meta_title': 'meta title',
                'keywords': 'django',
            })
        self.assertEqual(response.status_code, 200)
        self.assertFalse('Successfully added your page' in str(response.content))
        response = self.client.post(
            reverse('edit_page', kwargs={'page_slug': self.page.slug}),
            {
                'title': 'Hello world',
                'content': 'This is content',
                'meta_description': 'page meta data',
                'meta_title': 'meta title',
                'keywords': 'django',
            })
        self.assertEqual(response.status_code, 200)

    def test_delete_page(self):
        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)
        response = self.client.get(reverse('delete_page', kwargs={'page_slug': self.page.slug}))
        self.assertEqual(response.status_code, 302)

    def test_delete_page_404(self):
        user_login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(user_login)
        self.user.is_superuser = False
        self.user.save()
        response = self.client.get(reverse('delete_page', kwargs={'page_slug': self.page.slug}))
        self.assertEqual(response.status_code, 302)


class AdminLogin(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser('mp@mp.com', 'micro-test', 'mp')

    def test_admin_login(self):
        url = reverse('admin_login')
        self.client.login(username=self.user.email, password='mp')
        data = {'username': 'mp@mp.com', 'password': 'mp'}
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 200)
        self.user.is_active = False
        self.user.save()
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 200)
        data['email'] = 'not@used.com'
        response = response = self.client.post(url, data)
        self.assertEqual(response.status_code, 200)


class TestMenu(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser('mp@mp.com',
                                                  'mp',
                                                  'mp')

    def get_menu_and_parent(self):
        menu = Menu.objects.create(title='django',
                                   url="http://www.django.com",
                                   lvl=1)
        parent = Menu.objects.create(title='jquery',
                                     url="http://www.django.com",
                                     lvl=2)
        return menu, parent

    def test_add_menu(self):
        url = reverse('add_menu')
        is_logged_in = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(is_logged_in)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        response = self.client.post(url)
        self.assertEqual(response.status_code, 200)
        data = {'title': 'django', 'url': 'http://django.com'}
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 200)

    def test_edit_menu(self):
        menu, parent = self.get_menu_and_parent()
        url = reverse('edit_menu', kwargs={'pk': menu.pk})
        is_logged_in = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(is_logged_in)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        response = self.client.post(url)
        self.assertEqual(response.status_code, 200)
        context = {'title': 'django',
                   'url': 'http://django.com'}
        response = self.client.post(url, context)
        self.assertEqual(response.status_code, 200)
        context['parent'] = parent.pk
        response = self.client.post(url, context)
        self.assertEqual(response.status_code, 200)
        context['parent'] = menu.pk
        response = self.client.post(url, context)
        self.assertEqual(response.status_code, 200)
        menu.delete()
        parent.delete()


class Menus(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser('mp@mp.com',
                                                  'mp',
                                                  'mp')

    def test_menus(self):
        url = reverse('menus')
        is_logged_in = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(is_logged_in)
        context = {'select_status': 'True'}
        response = self.client.get(url, context)
        self.assertEqual(response.status_code, 200)
        context['select_status'] = 'False'
        response = self.client.get(url, context)
        self.assertEqual(response.status_code, 200)


class TestThemeCreateView(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser('mp@mp.com', 'mp', 'mp')

    def test_theme_create_view(self):
        is_logged_in = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(is_logged_in)
        url = reverse("add_theme")
        context = {
            "name": "theme-1",
            "enabled": "True",
            "description": "theme-1 description",
        }
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        response = self.client.post(url, context)
        self.assertEqual(response.status_code, 200)


class TestThemesList(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser('mp@mp.com', 'mp', 'mp')

    def test_themes_list(self):
        is_logged_in = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(is_logged_in)
        url = reverse("blog")
        context = {'select_status': 'Published', "search_text": "hi"}
        response = self.client.post(url, context)
        self.assertEqual(response.status_code, 200)


class TestMenuStatusUpdate(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser('mp@mp.com', 'mp', 'mp')
        self.menu = Menu.objects.create(title="menu-1", lvl=1)

    def test_menu_status_update(self):
        login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(login)
        url = reverse("menu_status_update", kwargs={"pk": self.menu.pk})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)


class TestThemesBulkActionsView(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser('mp@mp.com', 'mp', 'mp')
        Theme.objects.bulk_create([
            Theme(name="theme-1", slug="theme-1", description="desc", enabled=True),
            Theme(name="theme-2", slug="theme-2", description="desc", enabled=False),
            Theme(name="theme-3", slug="theme-3", description="desc", enabled=True),
            Theme(name="theme-4", slug="theme-4", description="desc", enabled=False),
        ])

    def test_bulk_actions_themes(self):
        login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(login)
        url = reverse("bulk_actions_themes")
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        context = {"theme_ids[]": list(Theme.objects.values_list("id", flat=True)), "action": "False"}
        response = self.client.get(url, context)
        self.assertEqual(response.status_code, 200)
        context = {"theme_ids[]": list(Theme.objects.values_list("id", flat=True)), "action": "Delete"}
        response = self.client.get(url, context)
        self.assertEqual(response.status_code, 200)


class TestThemeStatusUpdate(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser('mp@mp.com', 'mp', 'mp')
        self.theme = Theme.objects.create(name="theme-1", slug="theme-1", description="desc", enabled=True)

    def test_theme_status_change(self):
        login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(login)
        url = reverse("theme_status_update", kwargs={"theme_slug": self.theme.slug})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        url = reverse("theme_status_update", kwargs={"theme_slug": self.theme.slug})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)

    def test_delete_theme(self):
        login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(login)
        url = reverse("delete_theme", kwargs={"pk": self.theme.pk})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)


class TestThemeUpdateView(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser('mp@mp.com', 'mp', 'mp')
        self.theme = Theme.objects.create(name="theme-1", slug="theme-1", description="desc", enabled=True)

    def test_edit_theme(self):
        login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(login)
        url = reverse("edit_theme", kwargs={"pk": self.theme.pk})
        context = {
            "name": "theme-1",
            "description": "theme-1 descripiton",
            "enabled": True
        }
        response = self.client.post(url, context)
        self.assertEqual(response.status_code, 200)


class TestConfigureContactUs(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser('mp@mp.com', 'mp', 'mp')

    def test_configure_contact_us_get(self):
        login = self.client.login(username='mp@mp.com', password='mp')
        self.assertTrue(login)
        url = reverse("configure_contact_us")
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        context = {
            "from_email": "admin@mp.com",
            "reply_to_email": "admin1@mp.com",
            "email_admin": "admin2@mp.com",
            "subject": "Thank you for contacting us",
            "body_user": "Thanks! We will contact you soon",
            "body_admin": "Thanks"
        }
        response = self.client.post(url, context)
        self.assertEqual(response.status_code, 200)
        response = self.client.post(url, context)
        self.assertEqual(response.status_code, 200)


# test forms
class TestUserForm(TestCase):

    def setUp(self):
        self.user = User.objects.create_superuser('mp@mp.com', 'mp@mp.com', 'mp')

    def test_user_email(self):
        form = UserForm(data={"username": "mp@mp.com", "email": "mp@mp.com"})
        self.assertFalse(form.is_valid())

    def test_user_email_instance(self):
        form = UserForm(data={"username": "mp@mp.com", "email": "mp@mp.com"}, instance=self.user)
        self.assertFalse(form.is_valid())

    def test_user_save(self):
        form = UserForm(data={"username": "mp@mp.com", "email": "mp@mp.com", "password": "password", "role": "Admin", "code": "Admin"}, instance=self.user)
        self.assertTrue(form.is_valid())
        form.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-28 09:36
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_blog_it', '0006_page'),
    ]

    operations = [
        migrations.CreateModel(
            name='Menu',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255)),
                ('url', models.URLField(blank=True, max_length=255)),
                ('status', models.BooleanField(default=True)),
                ('lvl', models.IntegerField(blank=True)),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='django_blog_it.Menu')),
            ],
        ),
    ]



from django.http.response import HttpResponseRedirect
from .models import UserRole, Post
from django.shortcuts import render, render_to_response, get_object_or_404
from django.contrib import messages


def get_user_role(user):
    user_role = UserRole.objects.filter(user=user)
    if user_role:
        return user_role[0].role
    return 'No User Role'


class AdminOnlyMixin(object):

    def dispatch(self, request, *args, **kwargs):
        user = self.request.user
        if not (user.is_authenticated and user.is_active):
            return HttpResponseRedirect('/dashboard/')
        user_role = UserRole.objects.filter(user=request.user).last()
        if user_role:
            user_role = True if user_role.role == "Admin" else False
        else:
            user_role = False
        if not (user.is_superuser or user_role):
            messages.warning(request, "You don't have permission")
            return HttpResponseRedirect(request.META.get('HTTP_REFERER', "/"))
        return super(AdminOnlyMixin, self).dispatch(request, *args, **kwargs)


class AuthorNotAllowedMixin(object):

    def dispatch(self, request, *args, **kwargs):
        user = self.request.user
        if not (user.is_authenticated and user.is_active):
            return HttpResponseRedirect('/dashboard/')
        user_role = UserRole.objects.filter(user=request.user).last()
        if user_role:
            user_role = user_role.role in ["Admin", "Publisher"]
        if not (user.is_superuser or user_role):
            messages.warning(request, "You don't have permission")
            return HttpResponseRedirect(request.META.get('HTTP_REFERER', "/"))
        return super(AuthorNotAllowedMixin, self).dispatch(request, *args, **kwargs)


class AdminMixin(object):

    def dispatch(self, request, *args, **kwargs):
        user = self.request.user
        if not (user.is_authenticated and user.is_active):
            return HttpResponseRedirect('/dashboard/')
        # if not user.is_superuser:
        #     return HttpResponseRedirect("/")
        return super(AdminMixin, self).dispatch(request,
                                                *args,
                                                **kwargs)


class PostAccessRequiredMixin(object):

    def dispatch(self, request, *args, **kwargs):
        self.object = get_object_or_404(Post, slug=kwargs.get('blog_slug'))

        # Checking the permissions
        if not(
            self.object.is_deletable_by(request.user) or
            request.user.is_superuser is True or
            get_user_role(request.user) != 'Author'
        ):
            # TODO: Add "PermissionDenied" message
            messages.warning(request, "You don't have permission")
            return HttpResponseRedirect(request.META.get('HTTP_REFERER'))

        return super(PostAccessRequiredMixin, self).dispatch(
            request, *args, **kwargs)



from django.conf.urls import url
from .views import *

urlpatterns = [
    url(r'^$', admin_login, name='admin_login'),
    url(r'^logout/$', admin_logout, name='admin_logout'),

    # Blog
    url(r'^blog/$', PostList.as_view(), name='blog'),
    url(r'^add/$', PostCreatView.as_view(), name='blog_add'),
    url(r'^view/(?P<blog_slug>[-\w]+)/$', PostDetailView.as_view(), name='view_blog'),
    url(r'^delete/(?P<blog_slug>[-\w]+)/$', PostDeleteView.as_view(), name='delete_blog'),
    url(r'^edit/(?P<blog_slug>[-\w]+)/$', edit_blog, name='edit_blog'),

    # Categories
    url(r'^add_category/$', add_category, name='add_category'),
    url(r'^category/$', categories, name='categories'),
    url(r'^delete_category/(?P<category_slug>[-\w]+)/$', delete_category, name='delete_category'),
    url(r'^edit_category/(?P<category_slug>[-\w]+)/$', edit_category, name='edit_category'),

    url(r'^bulk_actions_menu/$', bulk_actions_menu, name='bulk_actions_menu'),
    url(r'^bulk_actions_blog/$', bulk_actions_blog, name='bulk_actions_blog'),
    url(r'^bulk_actions_category/$', bulk_actions_category, name='bulk_actions_category'),
    url(r'upload_photos/$', upload_photos, name='upload_photos'),
    url(r'recent_photos/$', recent_photos, name='recent_photos'),

    # menu management
    url(r'^menu/$', menus, name='menus'),
]



from django.test import TestCase
from django.test import Client
from django_blog_it.django_blog_it.models import Category, Post, Tags
from django.contrib.auth.models import User


class posts_views_get(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(
            'mp@mp.com', 'micro-test', 'mp')
        self.category = Category.objects.create(
            name='django', description='django desc', user=self.user, is_active=True)
        self.blogppost = Post.objects.create(
            title='other python introduction',
            user=self.user,
            content='This is content',
            category=self.category,
            status='Published',
            slug="other-python-introduction")
        self.tag = Tags.objects.create(name='testtag')
        self.blogppost.tags.add(self.tag)

    def test_blog_get(self):

        response = self.client.get('/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'posts/new_index.html')

        response = self.client.get('/blog/'+str(self.blogppost.slug)+'/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'posts/new_blog_view.html')

        response = self.client.get('/blog/category/'+str(self.category.slug)+'/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'posts/new_index.html')

        response = self.client.get('/blog/tags/'+str(self.tag.slug)+'/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'posts/new_index.html')

        response = self.client.get(
            '/blog/'+str(self.blogppost.updated_on.year)+'/'+str(self.blogppost.updated_on.month)+'/')
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'posts/new_index.html')



from django.conf import settings
import os

AWS_ENABLED = getattr(settings, 'AWS_ENABLED', False)
BASE_DIR = getattr(settings, 'BASE_DIR')
MEDIA_URL = settings.MEDIA_URL if settings.MEDIA_URL else '/media/'  # settings.MEDIA_URL if settings.MEDIA_URL else '/media/'
MEDIA_ROOT = settings.MEDIA_ROOT if settings.MEDIA_ROOT else os.path.join(BASE_DIR + '/media/')  # settings.MEDIA_ROOT if settings.MEDIA_ROOT else os.path.join(BASE_DIR + '/media/')

DISQUS_SHORTNAME = getattr(settings, 'DISQUS_SHORTNAME', '')

BLOG_TITLE = "My Django Blog"
BLOG_DESCRIPTION = "Rapid development of web applications"
BLOG_KEYWORDS = "django, python, webframework"
BLOG_AUTHOR = "Micropyramid"



from django import forms
from .models import Post, Category, Page, Menu, ContactUsSettings, ROLE_CHOICE, Theme
from django.template.defaultfilters import slugify
# for authentication
from django.contrib.auth import authenticate
from django.contrib.auth.models import User


class UserForm(forms.ModelForm):
    password = forms.CharField(required=False, widget=forms.PasswordInput)
    role = forms.ChoiceField(choices=ROLE_CHOICE, required=True)

    class Meta:
        model = User
        fields = ('username', 'first_name', 'last_name', 'email', 'password', 'is_active')

    def clean_email(self):
        email = self.cleaned_data.get('email')
        username = self.cleaned_data.get('username')
        if self.instance.id:
            if User.objects.filter(email=email, username=username).exclude(id=self.instance.id).count():
                raise forms.ValidationError(u'Email addresses must be unique.')
        else:
            if User.objects.filter(email=email).exclude(username=username).count():
                raise forms.ValidationError(u'Email addresses must be unique.')
        return email

    def __init__(self, *args, **kwargs):
        super(UserForm, self).__init__(*args, **kwargs)
        self.fields['email'].required = True
        for field in iter(self.fields):
            if max(enumerate(iter(self.fields)))[0] != field:
                self.fields[field].widget.attrs.update({
                    'class': 'form-control',
                    "placeholder": "Please enter your " + field.capitalize()
                })

    def save(self, commit=True):
        user = super(UserForm, self).save(commit=False)
        password = self.cleaned_data["password"]
        if password:
            user.set_password(password)
        if commit:
            user.save()
        return user


class AdminLoginForm(forms.Form):
    username = forms.CharField(max_length=254)
    password = forms.CharField(label="Password", widget=forms.PasswordInput)

    def clean(self):
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')
        if username and password:
            user = authenticate(username=username, password=password)
            if user is None:
                raise forms.ValidationError("Incorrect Login Details")

        return self.cleaned_data


class BlogPostForm(forms.ModelForm):
    tags = forms.CharField(label="Tags", max_length=300, required=False)

    class Meta:
        model = Post
        exclude = ('slug', 'user', 'tags')

    def __init__(self, *args, **kwargs):
        self.user_role = kwargs.pop('user_role', None)
        super(BlogPostForm, self).__init__(*args, **kwargs)

        if self.user_role == 'Author':
            del self.fields['status']

        for field in iter(self.fields):
            if field == 'tags':
                self.fields[field].widget.attrs.update({
                    'class': 'form-control myTags', "placeholder": "Please enter your Blog " + field.capitalize()
                })
            else:
                self.fields[field].widget.attrs.update({
                    'class': 'form-control', "placeholder": "Please enter your Blog " + field.capitalize()
                })
        self.fields['category'].queryset = Category.objects.filter(is_active=True)

    def clean_status(self):
        if self.user_role == "Author":
            raise forms.ValidationError("Admin and Publisher can change status only.")
        return self.cleaned_data.get("status")


class BlogCategoryForm(forms.ModelForm):
    class Meta:
        model = Category
        exclude = ('slug',)

    def clean_name(self):
        if not self.instance.id:
            if Category.objects.filter(slug=slugify(self.cleaned_data['name'])).exists():
                raise forms.ValidationError('Category with this Name already exists.')
        else:
            if Category.objects.filter(name__icontains=self.cleaned_data['name']).exclude(id=self.instance.id):
                raise forms.ValidationError('Category with this Name already exists.')

        return self.cleaned_data['name']

    def __init__(self, *args, **kwargs):
        super(BlogCategoryForm, self).__init__(*args, **kwargs)

        for field in iter(self.fields):
            if max(enumerate(iter(self.fields)))[0] != field:
                self.fields[field].widget.attrs.update({
                    'class': 'form-control', "placeholder": "Please enter your Category " + field.capitalize()
                })


class UserRoleForm(forms.Form):
    role = forms.CharField(max_length=10)


class PageForm(forms.ModelForm):
    class Meta:
        model = Page
        exclude = ('slug',)

    def __init__(self, *args, **kwargs):
        super(PageForm, self).__init__(*args, **kwargs)

        for field in iter(self.fields):
            if max(enumerate(iter(self.fields)))[0] != field:
                self.fields[field].widget.attrs.update({
                    'class': 'form-control', "placeholder": "Please enter your Page " + field.capitalize()
                })

    def clean_title(self):
        if not self.instance.id:
            if self.Meta.model.objects.filter(slug=slugify(self.cleaned_data['title'])).exists():
                raise forms.ValidationError('Page with this title already exists.')
        else:
            if self.Meta.model.objects.filter(title__icontains=self.cleaned_data['title']).exclude(id=self.instance.id):
                raise forms.ValidationError('Page with this title already exists.')

        return self.cleaned_data['title']


class MenuForm(forms.ModelForm):
    class Meta:
        model = Menu
        exclude = ('lvl',)

    def __init__(self, *args, **kwargs):
        super(MenuForm, self).__init__(*args, **kwargs)

        for field in iter(self.fields):
            if max(enumerate(iter(self.fields)))[0] != field:
                self.fields[field].widget.attrs.update({
                    'class': 'form-control', "placeholder": "Please enter your Menu " + field.capitalize()
                })


class ContactUsSettingsForm(forms.ModelForm):

    class Meta:
        model = ContactUsSettings
        exclude = ()

    def __init__(self, *args, **kwargs):
        super(ContactUsSettingsForm, self).__init__(*args, **kwargs)
        for field in iter(self.fields):
            if max(enumerate(iter(self.fields)))[0] != field:
                self.fields[field].widget.attrs.update({
                    'class': 'form-control',
                    "placeholder": "Please enter your " + field.replace('_', ' ').capitalize()
                })


class BlogThemeForm(forms.ModelForm):

    class Meta:
        model = Theme
        exclude = ('slug',)
        widgets = {
            'description': forms.Textarea(attrs={'class': 'form-control'}),
        }

    def __init__(self, *args, **kwargs):
        super(BlogThemeForm, self).__init__(*args, **kwargs)
        for field in iter(self.fields):
            self.fields[field].widget.attrs.update({
                'class': 'form-control',
                "placeholder": "Please enter your Theme " + field.capitalize()
            })


class ContactForm(forms.Form):
    contact_name = forms.CharField(max_length=128, required=True)
    contact_email = forms.EmailField(required=True)
    contact_website = forms.URLField(
        max_length=200, required=True, help_text="Enter Your Website URL here")
    content = forms.CharField(
        required=True,
        widget=forms.Textarea
    )

    def clean(self):
        cleaned_data = super(ContactForm, self).clean()
        contact_website = cleaned_data.get('contact_website')

        if contact_website and not contact_website.startswith('http://'):
            contact_website = 'http://' + contact_website
            cleaned_data['contact_website'] = contact_website

        return cleaned_data

    def __init__(self, *args, **kwargs):
        super(ContactForm, self).__init__(*args, **kwargs)
        self.fields['contact_name'].widget.attrs['placeholder'] = "Enter Your Name (Required)"
        self.fields['contact_email'].widget.attrs['placeholder'] = "Enter Your Email (Required)"
        self.fields['contact_website'].widget.attrs['placeholder'] = "Enter Your Website (Required)"
        self.fields['content'].widget.attrs['placeholder'] = "What do you want to say?"
        for field in iter(self.fields):
            self.fields[field].widget.attrs.update({
                'class': 'form-control'
            })


class ChangePasswordForm(forms.Form):
    password = forms.CharField(widget=forms.PasswordInput(attrs={"class": "form-control"}))
    confirm_password = forms.CharField(widget=forms.PasswordInput(attrs={"class": "form-control"}))

    def clean_confirm_password(self):
        password = self.cleaned_data.get("password")
        confirm_password = self.cleaned_data.get("confirm_password")
        if password != confirm_password:
            raise forms.ValidationError("Passwords do not match!!!")
        return confirm_password


class CustomBlogSlugInlineFormSet(forms.BaseInlineFormSet):
    def clean(self):
        super(CustomBlogSlugInlineFormSet, self).clean()
        if any(self.errors):
            return
        active_slugs = 0
        for form in self.forms:
            if form.cleaned_data.get("is_active"):
                active_slugs += 1
        if active_slugs > 1:
            raise forms.ValidationError("Only one slug can be active at a time.")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_blog_it', '0003_auto_20160224_0410'),
    ]

    operations = [
        migrations.CreateModel(
            name='PostHistory',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('content', models.CharField(max_length=200)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('post', models.ForeignKey(related_name='history', to='django_blog_it.Post', on_delete=models.CASCADE)),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)),
            ],
        ),
    ]



from django.conf.urls import url
from django.conf.urls.static import static
from .django_blog_it.views import *
from .posts.views import *
from .settings import MEDIA_URL, MEDIA_ROOT


urlpatterns = [
    url(r'^$', Home.as_view(), name='index'),
    url(r'^blog/contact/$', contact_us, name='contact_us'),
    url(r'^blog/category/(?P<category_slug>[-\w]+)/$', SelectedCategoryView.as_view(), name='selected_category'),
    url(r'^blog/tags/(?P<tag_slug>[-\w]+)/$', SelectedTagView.as_view(), name='selected_tag'),
    url(r'^blog/(?P<year>\w{0,})/(?P<month>\w{0,})/$', ArchiveView.as_view(), name='archive_posts'),
    url(r'^blog/(?P<blog_slug>[-\w]+)/$', BlogPostView.as_view(), name='blog_post_view'),

    url(r'^dashboard/$', AdminLoginView.as_view(), name='admin_login'),
    url(r'^dashboard/gplus/$', google_login, name='google_login'),
    url(r'^fb/$', facebook_login, name='facebook_login'),
    url(r'^dashboard/logout/$',
        admin_logout,
        name='admin_logout'),
    url(r'^dashboard/blog/$',
        PostList.as_view(),
        name='blog'),
    url(r'^dashboard/add/$',
        PostCreateView.as_view(),
        name='blog_add'),
    url(r'^dashboard/view/(?P<blog_slug>[-\w]+)/$',
        PostDetailView.as_view(),
        name='view_blog'),
    url(r'^dashboard/delete/(?P<blog_slug>[-\w]+)/$',
        PostDeleteView.as_view(),
        name='delete_blog'),
    url(r'^dashboard/edit/(?P<blog_slug>[-\w]+)/$',
        PostEditView.as_view(),
        name='edit_blog'),
    url(r'^dashboard/bulk_actions_blog/$',
        BlogPostBulkActionsView.as_view(),
        name='bulk_actions_blog'),

    url(r'^dashboard/category/$', CategoryList.as_view(), name='categories'),
    url(r'^dashboard/category/add/$', CategoryCreateView.as_view(), name='add_category'),
    url(r'^dashboard/category/edit/(?P<category_slug>[-\w]+)/$', CategoryUpdateView.as_view(), name='edit_category'),
    url(r'^dashboard/category/delete/(?P<category_slug>[-\w]+)/$', CategoryDeleteView.as_view(), name='delete_category'),
    url(r'^dashboard/category/status/(?P<category_slug>[-\w]+)/$', CategoryStatusUpdateView.as_view(), name='category_status_update'),
    # pages
    url(r'^dashboard/bulk_actions_category/$',
        CategoryBulkActionsView.as_view(), name='bulk_actions_category'),

    url(r'^dashboard/pages/$', PagesListView.as_view(), name='pages'),
    url(r'^dashboard/pages/add/$', PageCreateView.as_view(), name='add_page'),
    url(r'^dashboard/pages/edit/(?P<page_slug>[-\w]+)/$',
        PageUpdateView.as_view(), name='edit_page'),
    url(r'^dashboard/pages/update/(?P<page_slug>[-\w]+)/$',
        page_status_update, name='page_status_update'),
    url(r'^dashboard/pages/delete/(?P<page_slug>[-\w]+)/$',
        PageDeleteView.as_view(), name='delete_page'),
    url(r'^dashboard/bulk_actions_pages/$',
        BulkActionsPageView.as_view(), name='bulk_actions_pages'),
    url(r'^(?P<page_slug>[-\w]+)/$', PageView.as_view(), name='page_view'),

    url(r'^dashboard/upload_photos/$', upload_photos, name='upload_photos'),
    url(r'^dashboard/recent_photos/$', recent_photos, name='recent_photos'),
    url(r'^dashboard/users/$', UserListView.as_view(), name='users'),
    url(r'^dashboard/users/add/$', UserCreateView.as_view(), name='add_user'),
    url(r'^dashboard/user/edit/(?P<pk>[-\w]+)/$', UserUpdateView.as_view(), name='edit_user'),
    url(r'^dashboard/user/update/(?P<pk>[-\w]+)/$', user_status_update, name='user_status_update'),
    url(r'^dashboard/user/edit/(?P<pk>[-\w]+)/user_role/$',
        edit_user_role, name='edit_user_role'),
    url(r'^dashboard/user/delete/(?P<pk>[-\w]+)/$',
        UserDeleteView.as_view(), name='delete_user'),
    url(r'^dashboard/bulk_actions_users/$',
        UserBulkActionsView.as_view(), name='bulk_actions_users'),

    # menu management
    url(r'^dashboard/menu/$', MenuListView.as_view(), name='menus'),
    url(r'^dashboard/menu/add/$', MenuCreateView.as_view(), name='add_menu'),
    url(r'^dashboard/menu/edit/(?P<pk>[-\w]+)/$', MenuUpdateView.as_view(), name='edit_menu'),
    url(r'^dashboard/menu/update/(?P<pk>[-\w]+)/$', menu_status_update, name='menu_status_update'),
    url(r'^dashboard/bulk_actions_menu/$',
        MenuBulkActionsView.as_view(), name='bulk_actions_menu'),

    # themes management
    url(r'^dashboard/themes/$', ThemesList.as_view(), name='themes'),
    # url(r'^dashboard/themes/add/$', add_theme, name='add_theme'),
    url(r'^dashboard/themes/add/$',
        ThemeCreateView.as_view(),
        name='add_theme'),
    url(r'^dashboard/themes/(?P<theme_slug>[-\w]+)/$',
        ThemeDetailView.as_view(),
        name='view_theme'),
    url(r'^dashboard/themes/edit/(?P<pk>[0-9]+)/$',
        ThemeUpdateView.as_view(),
        name='edit_theme'),
    # url(r'^dashboard/themes/edit/(?P<theme_slug>[-\w]+)/$',
    #     edit_theme, name='edit_theme'),
    url(r'^dashboard/themes/delete/(?P<pk>[-\w]+)/$',
        DeleteThemeView.as_view(),
        name='delete_theme'),
    # url(r'^dashboard/themes/delete/(?P<theme_slug>[-\w]+)/$',
    #     delete_theme, name='delete_theme'),
    url(r'^dashboard/bulk_actions_themes/$',
        ThemesBulkActionsView.as_view(), name='bulk_actions_themes'),

    url(r'^dashboard/themes/update/(?P<theme_slug>[-\w]+)/$',
        theme_status_update,
        name='theme_status_update'),

    url(r'^dashboard/contactUs/$',
        configure_contact_us, name='configure_contact_us'),
    url(r'^dashboard/change-password/$', ChangePasswordView.as_view(), name='change_password'),

] + static(MEDIA_URL, document_root=MEDIA_ROOT)



import datetime
import os
from django import template
from django_blog_it.django_blog_it.models import Post, Tags, Menu
from django_blog_it.django_blog_it.views import get_user_role
from django_blog_it import settings

register = template.Library()


@register.simple_tag(takes_context=True)
def get_archives(context):
    archives = []
    dates = []
    for each_object in Post.objects.filter(category__is_active=True, status="Published").order_by('created_on').values('created_on'):
        for date in each_object.values():
            dates.append((date.year, date.month, 1))

    dates = list(set(dates))
    dates.sort(reverse=True)
    for each in dates:
        archives.append(datetime.datetime(each[0], each[1], each[2]))

    if len(archives) > 5:
        return archives[:5]
    return archives


@register.filter
def seperate_tags(tags):
    if tags:
        tags_list = tags.split(',')
        real_tags = []
        for tag in tags_list:
            real_tags.append(Tags.objects.get(name=tag))
        return real_tags
    return None


@register.filter
def is_deletable_by(blog_post, user):
    return blog_post.is_deletable_by(user)


@register.filter
def get_user_role_name(user):
    return get_user_role(user)


@register.filter
def get_range(value):
    return range(value)


@register.inclusion_tag('posts/new_nav_menu.html', takes_context=True)
def load_menu(context):
    context['menu'] = Menu.objects.filter(parent=None, status=True).order_by("lvl")
    return context


@register.filter
def posts_published_list(blogs_list):
    return len(blogs_list.filter(status='Published'))


@register.filter
def user_drafted_posts(user):
    return Post.objects.filter(user=user, status='Drafted').count()


@register.filter
def user_published_posts(user):
    return Post.objects.filter(user=user,
                               status='Published'
                               ).count()


@register.simple_tag
def blog_title():
    return settings.BLOG_TITLE


@register.filter
def category_posts(category):
    return Post.objects.filter(category=category).count()


@register.simple_tag
def google_analytics_id():
    return os.getenv("GOOGLE_ANALYTICS_ID")



# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))

SECRET_KEY = '_nc9_+ntllx!ptq5tzh96nn^$a#81#auhrdw#8t#+n0#4ig(lz'

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'django_blog_it.urls'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.humanize',
    'simple_pagination',
    'django_blog_it.django_blog_it',
    'django_blog_it.posts',
)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR + 'sqlite3.db'
    }
}
ALLOWED_HOSTS = ["127.0.0", "localhost"]
LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

STATIC_URL = '/static/'

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.request',
            ],
        },
    },
]
DEBUG = True
EMAIL_PORT    = 1025
EMAIL_HOST    = '127.0.0.1'
EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'



import uuid
import json
from PIL import Image
import os
import requests
from django.db.models.aggregates import Max
from django.shortcuts import render, render_to_response, get_object_or_404
from django.http import HttpResponseRedirect, HttpResponse, Http404
from django.contrib import messages
from django.contrib import auth
from django.contrib.auth import logout, authenticate, login, load_backend
from django.contrib.auth.decorators import user_passes_test, login_required
from django.views.decorators.csrf import csrf_exempt
from django.core.files import File
from django import forms
from django.forms import inlineformset_factory
from .models import Menu, Post, PostHistory, Category, Tags, Image_File, \
    STATUS_CHOICE, ROLE_CHOICE, UserRole, Page, Theme, Google, Facebook, \
    Post_Slugs
from .forms import *
from django.conf import settings
try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except ImportError:
    from django.contrib.auth.models import User
from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import reverse, reverse_lazy
from django.views.generic import ListView, DetailView, CreateView, DeleteView,\
    UpdateView, FormView, TemplateView, View
from django.views.generic.edit import ProcessFormView
from .mixins import AdminMixin, PostAccessRequiredMixin, AdminOnlyMixin, AuthorNotAllowedMixin
from django.http import JsonResponse

admin_required = user_passes_test(lambda user: user.is_active, login_url='/')


def active_admin_required(view_func):
    decorated_view_func = login_required(admin_required(view_func), login_url='/')
    return decorated_view_func


class AdminLoginView(FormView):
    template_name = "dashboard/new_admin-login.html"
    form_class = AdminLoginForm

    def dispatch(self, request, *args, **kwargs):
        if(request.user.is_authenticated and request.user.is_active):
            return HttpResponseRedirect(reverse_lazy("blog"))
        return super(AdminLoginView, self).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        user = authenticate(username=form.cleaned_data['username'],
                            password=form.cleaned_data['password'])
        if user.is_active:
            login(self.request, user)
            messages.success(self.request, 'You are successfully logged in')
            data = {'error': False,
                    'response': 'Successfully logged in'}
        else:
            data = {'error': True,
                    'response': 'You are not allowed to this page'}
        return JsonResponse(data)

    def form_invalid(self, form):
        return JsonResponse({"error": True, "errors": form.errors})


@active_admin_required
def admin_logout(request):
    logout(request)
    messages.success(request, 'You are successfully logged out!')
    return HttpResponseRedirect(reverse('admin_login'))


class PostList(AdminMixin, ListView):
    model = Post
    template_name = 'dashboard/blog/new_blog_list.html'
    context_object_name = 'blog_list'

    def get_context_data(self, **kwargs):
        context = super(PostList, self).get_context_data(**kwargs)
        context['blog_choices'] = STATUS_CHOICE
        context['object_list'] = self.model.objects.all()
        context['published_blogs'] = self.model.objects.filter(
            status='Published')
        return context

    def post(self, request, *args, **kwargs):
        blog_list = self.model.objects.all().order_by('id')

        if request.POST.get('select_status', ''):
            blog_list = blog_list.filter(
                status=request.POST.get('select_status')
            )
        if request.POST.get('search_text', ''):
            blog_list = list(set(blog_list.filter(
                title__icontains=request.POST.get('search_text')
            ) | blog_list.filter(
                tags__name__icontains=request.POST.get('search_text')
            )))
        return render(request, self.template_name,
                      {'blog_list': blog_list, 'blog_choices': STATUS_CHOICE})


class PostDetailView(DetailView):
    model = Post
    template_name = 'dashboard/blog/new_blog_view.html'
    slug_url_kwarg = "blog_slug"
    context_object_name = 'blog_post'


class PostCreateView(AdminMixin, CreateView):
    model = Post
    form_class = BlogPostForm
    template_name = "dashboard/blog/new_blog_add.html"
    success_url = reverse_lazy('blog')

    def get_form_kwargs(self):
        kwargs = super(PostCreateView, self).get_form_kwargs()
        role = get_user_role(self.request.user)
        role = role if role in dict(ROLE_CHOICE).keys() else None
        kwargs["user_role"] = role
        return kwargs

    def form_valid(self, form):
        self.blog_post = form.save(commit=False)
        formset = inlineformset_factory(
            Post, Post_Slugs, can_delete=True, extra=3, fields=('slug', 'is_active'),
            formset=CustomBlogSlugInlineFormSet)
        formset = formset(self.request.POST, instance=self.blog_post)
        if not formset.is_valid():
            return JsonResponse({'error': True, "response": formset.errors})
        self.blog_post.user = self.request.user
        if self.request.user.is_superuser:
            self.blog_post.status = self.request.POST.get('status')
        self.blog_post.save()
        self.blog_post.store_old_slug(self.blog_post.slug)
        formset.save()
        if self.request.POST.get('tags', ''):
            splitted = self.request.POST.get('tags').split(',')
            for s in splitted:
                blog_tags = Tags.objects.filter(name__iexact=s.strip())
                if blog_tags:
                    blog_tag = blog_tags.first()
                else:
                    blog_tag = Tags.objects.create(name=s.strip())
                self.blog_post.tags.add(blog_tag)

        self.blog_post.create_activity(user=self.request.user, content="added")
        messages.success(self.request, 'Successfully posted your blog')
        data = {'error': False, 'response': 'Successfully posted your blog',
                'title': self.request.POST['title']}
        return JsonResponse(data)

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})

    def get_context_data(self, **kwargs):
        context = super(PostCreateView, self).get_context_data(**kwargs)
        tags_list = Tags.objects.all()
        categories_list = Category.objects.filter(is_active=True)
        context['status_choices'] = STATUS_CHOICE
        context['categories_list'] = categories_list
        context['tags_list'] = tags_list
        context['add_blog'] = True
        self.formset = inlineformset_factory(
            Post, Post_Slugs, can_delete=True, extra=3, fields=('slug', 'is_active'),
            formset=CustomBlogSlugInlineFormSet,
            widgets={'slug': forms.TextInput(attrs={'class': 'form-control'})})
        context['formset'] = self.formset()
        return context


class PostEditView(AdminMixin, UpdateView):
    model = Post
    success_url = reverse_lazy('blog')
    slug_url_kwarg = 'blog_slug'
    template_name = "dashboard/blog/new_blog_add.html"
    form_class = BlogPostForm
    context_object_name = "blog_name"

    def dispatch(self, request, *args, **kwargs):
        if request.POST:
            instance = self.get_object()
            if request.POST.get("history_id"):
                history_post = instance.history.filter(id=request.POST.get("history_id")).last()
                if history_post:
                    return JsonResponse({"content": history_post.content})
        return super(PostEditView, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(PostEditView, self).get_form_kwargs()
        role = get_user_role(self.request.user)
        role = role if role in dict(ROLE_CHOICE).keys() else None
        kwargs["user_role"] = role
        return kwargs

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})

    def form_valid(self, form):
        formset = inlineformset_factory(
            Post, Post_Slugs, can_delete=True, extra=3, fields=('slug', 'is_active'), formset=CustomBlogSlugInlineFormSet)
        formset = formset(self.request.POST, instance=self.get_object())
        if not formset.is_valid():
            return JsonResponse({'error': True, "response": formset.errors})
        else:
            formset.save()
        previous_status = self.get_object().status
        previous_content = self.get_object().content
        self.blog_post = form.save(commit=False)
        # self.blog_post.user = self.request.user
        if self.request.user.is_superuser or get_user_role(self.request.user) != 'Author':
            self.blog_post.status = self.request.POST.get('status')
        else:
            self.blog_post.status = previous_status
        self.blog_post.save()
        self.blog_post.tags.clear()
        if self.request.POST.get('tags', ''):
            splitted = self.request.POST.get('tags').split(',')
            for s in splitted:
                blog_tags = Tags.objects.filter(name__iexact=s.strip())
                if blog_tags:
                    blog_tag = blog_tags.first()
                else:
                    blog_tag = Tags.objects.create(name=s.strip())
                self.blog_post.tags.add(blog_tag)
        if previous_content != self.blog_post.content:
            self.blog_post.create_activity(
                user=self.request.user, content=previous_content)
        # if self.blog_post.status == previous_status:
        #     self.blog_post.create_activity(
        #         user=self.request.user, content="updated")
        # else:
        #     self.blog_post.create_activity(
        #         user=self.request.user,
        #         content="changed status from " +
        #         str(previous_status) + " to " + str(blog_post.status)
        #     )
        self.blog_post.store_old_slug(self.kwargs.get("blog_slug"))
        messages.success(self.request, 'Successfully updated your blog post')
        data = {'error': False,
                'response': 'Successfully updated your blog post'}
        return JsonResponse(data)

    def get_context_data(self, **kwargs):
        context = super(PostEditView, self).get_context_data(**kwargs)
        categories_list = Category.objects.filter(is_active=True)
        context['status_choices'] = STATUS_CHOICE,
        context['categories_list'] = categories_list
        context['history_list'] = self.get_object().history.all()
        self.formset = inlineformset_factory(
            Post, Post_Slugs, can_delete=True, extra=3, fields=('slug', 'is_active'),
            formset=CustomBlogSlugInlineFormSet,
            widgets={'slug': forms.TextInput(attrs={'class': 'form-control'})})
        context['formset'] = self.formset(instance=self.get_object())
        return context


class PostDeleteView(PostAccessRequiredMixin, DeleteView):
    model = Post
    success_url = reverse_lazy("blog")
    slug_field = 'slug'
    template_name = "dashboard/blog/new_blog_list.html"

    def get_object(self):
        return get_object_or_404(Post, slug=self.kwargs['blog_slug'])

    def post(self, request, *args, **kwargs):
        blog_post = self.get_object()
        previous_status = blog_post.status
        if request.POST.get("action") == "trash":
            blog_post.status = "Trashed"
            blog_post.save()
            blog_post.create_activity(
                user=request.user,
                content="moved to trash from " + str(previous_status)
            )
            messages.success(
                request,
                'Blog "' + str(blog_post.title) + '" has been moved to trash.'
            )
            return HttpResponseRedirect(self.success_url)
        elif request.POST.get("action") == "restore":
            blog_post.status = "Drafted"
            blog_post.save()
            blog_post.create_activity(
                user=request.user,
                content="restored from trash to " + str(blog_post.status)
            )
            messages.success(
                request,
                'Blog "' + str(blog_post.title) + '" has been restored from trash.'
            )
            return HttpResponseRedirect(self.success_url)
        elif request.POST.get("action") == "delete":
            blog_post.remove_activity()
            blog_post.delete()
            messages.success(request, 'Blog successfully deleted')
            return HttpResponseRedirect(self.success_url)
        else:
            raise Http404


class CategoryList(AdminMixin, TemplateView, ProcessFormView):
    template_name = "dashboard/category/new_categories_list.html"

    def post(self, request, *args, **kwargs):
        return render(request, self.template_name, self.get_context_data())

    def get_queryset(self):
        self.category_choices = Category.objects.all()
        queryset = self.category_choices
        if self.request.POST.get('select_status'):
            if self.request.POST.get('select_status') == "True":
                queryset = queryset.filter(is_active=True)
            else:
                queryset = queryset.filter(is_active=False)
        if self.request.POST.get('search_text'):
            queryset = queryset.filter(name__icontains=self.request.POST.get('search_text'))
        return queryset

    def get_context_data(self, *args, **kwargs):
        context = {
            "categories_list": self.get_queryset(),
            'requested_categories': self.request.POST.getlist('category'),
            'category_choices': self.category_choices,
        }
        return context


class CategoryCreateView(AdminOnlyMixin, CreateView):
    template_name = "dashboard/category/new_category_add.html"
    form_class = BlogCategoryForm

    def form_valid(self, form):
        form.save()
        messages.success(self.request, 'Successfully added your category')
        return JsonResponse({'error': False, 'response': 'Successfully added your category'})

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


class CategoryUpdateView(AdminOnlyMixin, UpdateView):
    template_name = "dashboard/category/new_category_add.html"
    model = Category
    slug_url_kwarg = "category_slug"
    form_class = BlogCategoryForm

    def form_valid(self, form):
        form.save()
        messages.success(self.request, 'Successfully updated your category')
        return JsonResponse({'error': False, 'response': 'Successfully updated your category'})

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


class CategoryStatusUpdateView(AdminOnlyMixin, View):

    def get(self, request, *args, **kwargs):
        category = get_object_or_404(Category, slug=kwargs.get("category_slug"))
        if category.is_active:
            category.is_active = False
        else:
            category.is_active = True
        category.save()
        return HttpResponseRedirect(reverse_lazy("categories"))


class CategoryDeleteView(AdminOnlyMixin, View):

    def get(self, request, *args, **kwargs):
        category = get_object_or_404(Category, slug=kwargs.get("category_slug"))
        category.delete()
        return HttpResponseRedirect(reverse_lazy("categories"))


class BlogPostBulkActionsView(AuthorNotAllowedMixin, View):

    def get(self, request, *args, **kwargs):
        if 'blog_ids[]' in request.GET:
            blog_posts = Post.objects.filter(id__in=request.GET.getlist('blog_ids[]'))
            if request.GET.get('action') in [status[0] for status in STATUS_CHOICE]:
                blog_posts.update(status=request.GET.get('action'))
                messages.success(request, "successfully updated status to " + request.GET.get('action'))
            elif request.GET.get('action') == 'Delete':
                PostHistory.objects.filter(post__in=blog_posts).delete()
                blog_posts.delete()
            return HttpResponse(json.dumps({'response': True}))
        else:
            messages.warning(request, 'Please select a record to perform action')
            return HttpResponse(json.dumps({'response': False}))


class CategoryBulkActionsView(AdminOnlyMixin, View):

    def get(self, request, *args, **kwargs):
        if 'blog_ids[]' in request.GET:
            if request.GET.get('action') == 'True':
                Category.objects.filter(id__in=request.GET.getlist('blog_ids[]')).update(is_active=True)
                messages.success(request, 'Selected Categories successfully updated as Active')
            elif request.GET.get('action') == 'False':
                Category.objects.filter(id__in=request.GET.getlist('blog_ids[]')).update(is_active=False)
                messages.success(request, 'Selected Categories successfully updated as Inactive')
            elif request.GET.get('action') == 'Delete':
                Category.objects.filter(id__in=request.GET.getlist('blog_ids[]')).delete()
                messages.success(request, 'Selected Categories successfully deleted!')
            return JsonResponse({'response': True})
        else:
            messages.warning(request, 'Please select a record to perform this action')
            return JsonResponse({'response': False})


@csrf_exempt
def upload_photos(request):
    '''
    takes all the images coming from the redactor editor and
    stores it in the database and returns all the files
    '''
    upurl = ''
    if request.FILES.get("upload"):
        f = request.FILES.get("upload")
        obj = Image_File.objects.create(upload=f, is_image=True)
        obj.save()
        thumbnail_name = 'thumb' + f.name
        if getattr(settings, 'AWS_ENABLED', False):
            image_file = requests.get(obj.upload.url, stream=True)
            with open(thumbnail_name, 'wb') as destination:
                for chunk in image_file.iter_content():
                    destination.write(chunk)
        else:
            image_file = f
            with open(thumbnail_name, 'wb') as destination:
                for chunk in image_file.chunks():
                    destination.write(chunk)
        im = Image.open(destination.name)
        size = (128, 128)
        im.thumbnail(size)
        im.save(thumbnail_name)
        with open(thumbnail_name, 'rb') as imdata:
            obj.thumbnail.save(thumbnail_name, File(imdata))
        obj.save()
        os.remove(os.path.join(settings.BASE_DIR, thumbnail_name))
        upurl = "/" + obj.upload.url
    return HttpResponse(
        """<script type='text/javascript'>
        window.parent.CKEDITOR.tools.callFunction({0}, '{1}');
        </script>""".format(request.GET['CKEditorFuncNum'], upurl)
    )


def get_user_role(user):
    user_role = UserRole.objects.filter(user=user)
    if user_role:
        return user_role[0].role
    return 'No User Role'


@csrf_exempt
def recent_photos(request):
    ''' returns all the images from the data base '''
    imgs = []
    for obj in Image_File.objects.filter(is_image=True).order_by("-date_created"):
        upurl = "/" + obj.upload.url
        thumburl = ""
        if obj.thumbnail:
            thumburl = "/" + obj.thumbnail.url
        imgs.append({'src': upurl, 'thumb': thumburl, 'is_image': True})
    return render_to_response('dashboard/browse.html', {'files': imgs})


class UserListView(AdminOnlyMixin, ListView):
    template_name = "dashboard/user/new_list.html"
    context_object_name = "users_list"

    def get_queryset(self):
        queryset = User.objects.all()
        if self.request.GET.get('select_role'):
            queryset = queryset.filter(userrole__role=self.request.GET.get('select_role'))
        if self.request.GET.get('search_text'):
            queryset = queryset.filter(
                username__icontains=self.request.GET.get('search_text')
            ) | queryset.filter(
                email__icontains=self.request.GET.get('search_text')
            ) | queryset.filter(
                first_name__icontains=self.request.GET.get('search_text')
            ) | queryset.filter(
                last_name__icontains=self.request.GET.get('search_text')
            )
        return queryset.order_by('-id')

    def get_context_data(self, *args, **kwargs):
        context = super(UserListView, self).get_context_data(*args, **kwargs)
        context['roles'] = ROLE_CHOICE
        return context


class UserCreateView(AdminOnlyMixin, CreateView):
    template_name = "dashboard/user/new_add_user.html"
    form_class = UserForm

    def form_valid(self, form):
        user = form.save()
        UserRole.objects.create(user=user, role=form.cleaned_data.get('role'))
        messages.success(self.request, 'Successfully added your User')
        return JsonResponse({"error": False, "response": "Successfully added your User"})

    def form_invalid(self, form):
        return JsonResponse({"error": True, "response": form.errors})


class UserUpdateView(AdminOnlyMixin, UpdateView):
    template_name = "dashboard/user/new_add_user.html"
    model = User
    form_class = UserForm
    pk = "pk"

    def get_initial(self):
        if self.object.userrole_set.last():
            return {'role': self.object.userrole_set.last().role}
        return {}

    def form_valid(self, form):
        user = form.save()
        urole = UserRole.objects.filter(user=user).last()
        if urole:
            urole.role = form.cleaned_data.get('role')
            urole.save()
        else:
            UserRole.objects.create(user=user, role=form.cleaned_data.get('role'))
        messages.success(self.request, 'Successfully Updated User "' + str(user) + '"')
        return JsonResponse({'error': False, 'response': 'Successfully Updated User "' + str(user) + '"'})

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


@active_admin_required
def user_status_update(request, pk):
    user = get_object_or_404(User, pk=pk)
    user_role = UserRole.objects.filter(user=request.user).last()
    if user_role:
        user_role = True if user_role.role == "Admin" else False
    else:
        user_role = False
    if request.user.is_superuser or user_role:
        if user.is_active:
            user.is_active = False
        else:
            user.is_active = True
        user.save()
    else:
        messages.warning(request, "You don't have permission")
    return HttpResponseRedirect(reverse_lazy("users"))


class UserDeleteView(AdminOnlyMixin, View):

    def get(self, request, *args, **kwargs):
        user = get_object_or_404(User, id=kwargs.get("pk"))
        user.delete()
        messages.success(request, 'User successfully deleted!')
        return HttpResponseRedirect(reverse_lazy("users"))


class UserBulkActionsView(AdminOnlyMixin, View):

    def get(self, request, *args, **kwargs):
        if 'user_ids[]' in request.GET:
            if request.GET.get('action') == 'True':
                User.objects.filter(id__in=request.GET.getlist('user_ids[]')).update(is_active=True)
                messages.success(request, 'Selected Users successfully updated as Active')
            elif request.GET.get('action') == 'False':
                User.objects.filter(id__in=request.GET.getlist('user_ids[]')).update(is_active=False)
                messages.success(request, 'Selected Users successfully updated as Inactive')
            elif request.GET.get('action') == 'Delete':
                User.objects.filter(id__in=request.GET.getlist('user_ids[]')).delete()
                messages.success(request, 'Selected Users successfully deleted!')
            return JsonResponse({'response': True})
        else:
            messages.warning(request, 'Please select a record to perform the action')
            return JsonResponse({'response': False})


def edit_user_role(request, pk):
    user_role = UserRole.objects.filter(user_id=pk)
    if request.method == 'GET':
        context = {'user_role': user_role, 'roles': ROLE_CHOICE}
        return render(request, 'dashboard/user/new_user_role.html', context)
    validate_user_role = UserRoleForm(request.POST)
    if validate_user_role.is_valid():
        if user_role:
            user_role = user_role[0]
            user_role.role = request.POST.get('role')
            user_role.save()
        else:
            user = User.objects.get(pk=pk)
            UserRole.objects.create(user=user, role=request.POST.get('role'))
        messages.success(request, 'Successfully Updated User Role')
        data = {'error': False, 'response': 'Successfully Updated User Role'}
    else:
        data = {'error': True, 'response': validate_user_role.errors}
    return HttpResponse(json.dumps(data))


class PagesListView(AdminMixin, ListView):
    template_name = "dashboard/pages/new_list.html"
    context_object_name = "pages_list"

    def get_queryset(self):
        queryset = Page.objects.all()
        if self.request.GET.get('select_status'):
            if self.request.GET.get('select_status') == "True":
                queryset = queryset.filter(is_active=True)
            else:
                queryset = queryset.filter(is_active=False)
        if self.request.GET.get('search_text'):
            queryset = queryset.filter(title__icontains=self.request.GET.get('search_text'))
        return queryset


class PageCreateView(AdminOnlyMixin, CreateView):
    template_name = "dashboard/pages/new_add_page.html"
    form_class = PageForm

    def form_valid(self, form):
        form.save()
        messages.success(self.request, 'Successfully added your page')
        return JsonResponse({'error': False, 'response': 'Successfully added your page'})

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


class PageUpdateView(AdminOnlyMixin, UpdateView):
    template_name = "dashboard/pages/new_add_page.html"
    model = Page
    form_class = PageForm
    slug_url_kwarg = "page_slug"

    def form_valid(self, form):
        form.save()
        messages.success(self.request, 'Successfully updated your page')
        return JsonResponse({'error': False, 'response': 'Successfully updated your page'})

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


@active_admin_required
def page_status_update(request, page_slug):
    page = get_object_or_404(Page, slug=page_slug)
    if page.is_active:
        page.is_active = False
    else:
        page.is_active = True
    page.save()
    return HttpResponseRedirect(reverse_lazy("pages"))


class PageDeleteView(AdminOnlyMixin, View):

    def get(self, request, *args, **kwargs):
        page = get_object_or_404(Page, slug=kwargs.get("page_slug"))
        page.delete()
        messages.success(request, 'Page successfully deleted!')
        return HttpResponseRedirect(reverse_lazy("pages"))


class BulkActionsPageView(AdminOnlyMixin, View):

    def get(self, request, *args, **kwargs):
        if 'page_ids[]' in request.GET:
            if request.GET.get('action') == 'True':
                Page.objects.filter(id__in=request.GET.getlist('page_ids[]')).update(is_active=True)
                messages.success(request, 'Selected Pages successfully updated as Active')
            elif request.GET.get('action') == 'False':
                Page.objects.filter(id__in=request.GET.getlist('page_ids[]')).update(is_active=False)
                messages.success(request, 'Selected Pages successfully updated as Inactive')
            elif request.GET.get('action') == 'Delete':
                Page.objects.filter(id__in=request.GET.getlist('page_ids[]')).delete()
                messages.success(request, 'Selected Pages successfully deleted!')
            return JsonResponse({'response': True})
        else:
            messages.warning(request, 'Please select a record to perform this action')
            return JsonResponse({'response': False})


class MenuListView(AdminMixin, ListView):
    template_name = "dashboard/menu/new_list.html"
    context_object_name = "menu_list"

    def get_queryset(self):
        queryset = Menu.objects.filter(parent=None)
        if self.request.GET.get('select_status'):
            if self.request.GET.get('select_status') == "True":
                queryset = queryset.filter(status=True)
            else:
                queryset = queryset.filter(status=False)
        if self.request.GET.get('search_text'):
            queryset = queryset.filter(title__icontains=self.request.GET.get('search_text'))
        return queryset


class MenuCreateView(AdminOnlyMixin, CreateView):
    template_name = "dashboard/menu/new_manage.html"
    form_class = MenuForm

    def form_valid(self, form):
        menu_obj = form.save(commit=False)
        menu_count = Menu.objects.filter(parent=menu_obj.parent).count()
        menu_obj.lvl = menu_count + 1
        if menu_obj.url[-1] != '/':
            menu_obj.url = menu_obj.url + '/'
        menu_obj.save()
        messages.success(self.request, 'Successfully added menu.')
        return JsonResponse({'error': False, 'response': 'Successfully added menu.'})

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


class MenuUpdateView(AdminOnlyMixin, UpdateView):
    template_name = "dashboard/menu/new_manage.html"
    model = Menu
    pk = "pk"
    form_class = MenuForm

    def form_valid(self, form):
        current_parent = self.object.parent
        current_lvl = self.object.lvl
        updated_menu_obj = form.save(commit=False)
        if updated_menu_obj.parent != current_parent:
            if updated_menu_obj.parent.id == updated_menu_obj.id:
                return JsonResponse({'error': True, 'message': 'you can not choose the same as parent'})
            menu_count = Menu.objects.filter(parent=updated_menu_obj.parent).count()
            updated_menu_obj.lvl = menu_count + 1
            menu_max_lvl = Menu.objects.filter(parent=current_parent).aggregate(Max('lvl'))['lvl__max']
            if menu_max_lvl != 1:
                for i in Menu.objects.filter(parent=current_parent, lvl__gt=current_lvl, lvl__lte=menu_max_lvl):
                    i.lvl = i.lvl - 1
                    i.save()
        if updated_menu_obj.url[-1] != '/':
            updated_menu_obj.url = updated_menu_obj.url + '/'
        updated_menu_obj.save()

        messages.success(self.request, 'Successfully updated menu')
        return JsonResponse({'error': False, 'response': 'Successfully updated menu'})

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


@active_admin_required
def menu_status_update(request, pk):
    menu = get_object_or_404(Menu, pk=pk)
    if menu.status:
        menu.status = False
    else:
        menu.status = True
    menu.save()
    return HttpResponseRedirect(reverse_lazy("menus"))


class MenuBulkActionsView(AdminOnlyMixin, View):

    def get(self, request, *args, **kwargs):
        if 'menu_ids[]' in request.GET:
            if request.GET.get('action') == 'True':
                Menu.objects.filter(id__in=request.GET.getlist('menu_ids[]')).update(status=True)
                messages.success(request, "Selected Menu's successfully updated as Active")
            elif request.GET.get('action') == 'False':
                Menu.objects.filter(id__in=request.GET.getlist('menu_ids[]')).update(status=False)
                messages.success(request, "Selected Menu's successfully updated as Inactive")
            elif request.GET.get('action') == 'Delete':
                Menu.objects.filter(id__in=request.GET.getlist('menu_ids[]')).delete()
                messages.success(request, "Selected Menu's successfully deleted!")
            return JsonResponse({'response': True})
        else:
            messages.warning(request, 'Please select a record to perform this action')
            return JsonResponse({'response': False})


@active_admin_required
def configure_contact_us(request):
    contact_us_settings = ContactUsSettings.objects.all().last()
    if request.method == 'POST':
        if contact_us_settings:
            form = ContactUsSettingsForm(instance=contact_us_settings,
                                         data=request.POST)
        else:
            form = ContactUsSettingsForm(request.POST)

        if form.is_valid():
            form.save()
            messages.success(
                request, 'Successfully saved your contact us details.')
            data = {'error': False,
                    'response': 'Successfully saved your contact us details.'}
        else:
            data = {'error': True, 'response': form.errors}
        return HttpResponse(json.dumps(data))
    else:
        if contact_us_settings:
            form = ContactUsSettingsForm(instance=contact_us_settings)
        else:
            form = ContactUsSettingsForm()
    context = {'form': form}
    return render(request, 'dashboard/new_contact_us_settings.html', context)


class ThemesList(AdminMixin, ListView):
    model = Theme
    template_name = 'dashboard/themes/themes_list.html'
    context_object_name = 'themes_list'

    def get_context_data(self, **kwargs):
        context = super(ThemesList, self).get_context_data(**kwargs)
        context['object_list'] = self.model.objects.all()
        context['enabled_themes'] = self.model.objects.filter(
            enabled=True)
        return context

    def post(self, request, *args, **kwargs):
        themes_list = self.model.objects.all()

        if request.POST.get('select_status', ''):
            if request.POST.get('select_status') == 'True':
                themes_list = themes_list.filter(
                    enabled=True
                )
            else:
                themes_list = themes_list.filter(
                    enabled=False
                )
        if request.POST.get('search_text', ''):
            themes_list = themes_list.filter(
                name__icontains=request.POST.get('search_text')
            )
        return render(request, self.template_name,
                      {'themes_list': themes_list})


class ThemeDetailView(AdminOnlyMixin, DetailView):
    model = Theme
    template_name = 'dashboard/themes/theme_view.html'
    slug_field = "theme_slug"
    context_object_name = 'theme'

    def get_object(self):
        return get_object_or_404(Theme, slug=self.kwargs['theme_slug'])


class ThemeCreateView(AdminOnlyMixin, CreateView):
    model = Theme
    form_class = BlogThemeForm
    template_name = "dashboard/themes/theme_add.html"
    success_url = reverse_lazy('themes')

    def form_valid(self, form):
        self.blog_theme = form.save(commit=False)
        if self.request.user.is_superuser:
            if self.request.POST.get('enabled') == 'True':
                Theme.objects.filter(
                    enabled=True).update(enabled=False)
                self.blog_theme.enabled = True
            elif self.request.POST.get('enabled') == 'False':
                self.blog_theme.enabled = False
        self.blog_theme.save()
        messages.success(self.request, 'Successfully Created your Theme')
        data = {'error': False, 'response': 'Successfully Created your Theme',
                'title': self.request.POST['name']}
        return JsonResponse(data)

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})

    def get_context_data(self, **kwargs):
        context = super(ThemeCreateView, self).get_context_data(**kwargs)
        form = BlogThemeForm(self.request.GET)
        context['form'] = form
        context['add_theme'] = True
        return context


class ThemeUpdateView(AdminOnlyMixin, UpdateView):
    pk = 'pk'
    model = Theme
    form_class = BlogThemeForm
    template_name = "dashboard/themes/theme_add.html"
    success_url = reverse_lazy('themes')

    def form_valid(self, form):
        blog_theme = form.save(commit=False)
        if self.request.user.is_superuser:
            if self.request.POST.get('enabled') == 'True':
                Theme.objects.filter(
                    enabled=True).update(enabled=False)
                blog_theme.enabled = True
            elif self.request.POST.get('enabled') == 'False':
                blog_theme.enabled = False
        blog_theme.save()
        messages.success(self.request, 'Successfully Updated your Theme')
        data = {'error': False, 'response': 'Successfully Updated your Theme',
                'title': self.request.POST['name']}
        return JsonResponse(data)

    def form_invalid(self, form):
        return JsonResponse({'error': True, 'response': form.errors})


@active_admin_required
def theme_status_update(request, theme_slug):
    blog_theme = get_object_or_404(Theme, slug=theme_slug)
    if blog_theme.enabled is True:
        blog_theme.enabled = False
    else:
        Theme.objects.filter(
            enabled=True).update(enabled=False)
        blog_theme.enabled = True
    blog_theme.save()
    return HttpResponseRedirect(reverse_lazy("themes"))


class DeleteThemeView(AdminOnlyMixin, View):

    def get(self, request, *args, **kwargs):
        theme = get_object_or_404(Theme, id=kwargs.get('pk'))
        theme.delete()
        return HttpResponseRedirect(reverse_lazy('themes'))


class ThemesBulkActionsView(AdminOnlyMixin, View):

    def get(self, request, *args, **kwargs):
        if 'theme_ids[]' in request.GET:
            if request.GET.get('action') == 'False':
                Theme.objects.filter(id__in=request.GET.getlist('theme_ids[]')).update(enabled=False)
                messages.success(request, "Selected Theme's successfully updated as Disabled")
            elif request.GET.get('action') == 'Delete':
                Theme.objects.filter(id__in=request.GET.getlist('theme_ids[]')).delete()
                messages.success(request, "Selected Theme's successfully deleted!")
            return JsonResponse({'response': True})
        else:
            messages.warning(request,'Please select at-least one record to perform this action')
            return JsonResponse({'response': False})

# social login
def google_login(request):
    if 'code' in request.GET:
        params = {
            'grant_type': 'authorization_code',
            'code': request.GET.get('code'),
            'redirect_uri': request.scheme + "://" + request.META['HTTP_HOST'] + reverse('google_login'),
            'client_id': os.getenv("GP_CLIENT_ID"),
            'client_secret': os.getenv("GP_CLIENT_SECRET")
        }
        info = requests.post("https://accounts.google.com/o/oauth2/token", data=params)
        info = info.json()
        url = 'https://www.googleapis.com/oauth2/v1/userinfo'
        params = {'access_token': info['access_token']}
        kw = dict(params=params, headers={}, timeout=60)
        response = requests.request('GET', url, **kw)
        user_document = response.json()
        link = "https://plus.google.com/" + user_document['id']
        picture = user_document['picture'] if 'picture' in user_document.keys() else ""
        dob = user_document['birthday'] if 'birthday' in user_document.keys() else ""
        gender = user_document['gender'] if 'gender' in user_document.keys() else ""
        link = user_document['link'] if 'link' in user_document.keys() else link

        if request.user.is_authenticated():
            user = request.user
        else:
            user = User.objects.filter(email=user_document['email']).first()
        if user:
            user.first_name = user_document['name']
            user.last_name = user_document['family_name']
            user.dob = dob
            user.save()
        else:
            user = User.objects.create(
                username=user_document['email'],
                email=user_document['email'],
                first_name=user_document['name'],
                last_name=user_document['family_name'],
            )
            user.set_password(uuid.uuid4())
        user.save()
        google, created = Google.objects.get_or_create(user=user)
        google.user = user
        google.google_url = link
        google.verified_email = user_document['verified_email']
        google.google_id = user_document['id']
        google.family_name = user_document['family_name']
        google.name = user_document['name']
        google.given_name = user_document['given_name']
        google.dob = dob
        google.email = user_document['email']
        google.gender = gender
        google.picture = picture
        google.save()

        if created:
            role = UserRole.objects.filter(user=user).last()
            if not role and user.is_superuser:
                UserRole.objects.create(user=user, role="Admin")
            elif not role:
                UserRole.objects.create(user=user, role="Author")
        if not request.user.is_authenticated():
            if not hasattr(user, 'backend'):
                for backend in settings.AUTHENTICATION_BACKENDS:
                    if user == load_backend(backend).get_user(user.pk):
                        user.backend = backend
                        break
            if hasattr(user, 'backend'):
                auth.login(request, user)
        messages.success(request, "Loggedin successfully")
        return HttpResponseRedirect(reverse_lazy('blog'))

    else:
        rty = "https://accounts.google.com/o/oauth2/auth?client_id=" + os.getenv("GP_CLIENT_ID")\
              + "&response_type=code"
        rty += "&scope=https://www.googleapis.com/auth/userinfo.profile \
               https://www.googleapis.com/auth/userinfo.email&redirect_uri=" + request.scheme\
               + "://" + request.META['HTTP_HOST'] + reverse('google_login')\
               + "&state=1235dfghjkf123"
        return HttpResponseRedirect(rty)


def facebook_login(request):
    if 'code' in request.GET:
        accesstoken = get_access_token_from_code(request.GET['code'], 'https://' + request.META['HTTP_HOST'] + reverse('facebook_login'), os.getenv("FB_APP_ID"), os.getenv("FB_SECRET"))
        if 'error' in accesstoken.keys():
            messages.error(request, "Sorry, Your session has been expired")
            return render(request, '404.html')
        graph = GraphAPI(accesstoken['access_token'])
        accesstoken = graph.extend_access_token(os.getenv("FB_APP_ID"), os.getenv("FB_SECRET"))['accesstoken']
        hometown = profile['hometown']['name'] if 'hometown' in profile.keys() else ''
        location = profile['location']['name'] if 'location' in profile.keys() else ''
        bday = datetime.strptime(profile['birthday'], '%m/%d/%Y').strftime('%Y-%m-%d') if 'birthday' in profile.keys() else '1970-09-09'

        if 'email' in profile.keys():
            user, created = User.objects.get_or_create(
                username=profile['email'],
                email=profile['email'],
                first_name=profile['first_name'],
                last_name=profile['last_name'],
                last_login=timezone.now()
            )
            if created:
                role = UserRole.objects.filter(user=user).last()
                if not role and user.is_superuser:
                    UserRole.objects.create(user=user, role="Admin")
                elif not role:
                    UserRole.objects.create(user=user, role="Author")
            fb = Facebook.objects.filter(user=user).last()
            if not fb:
                Facebook.objects.create(
                    user=user,
                    facebook_url=profile['link'],
                    facebook_id=profile['id'],
                    first_name=profile['first_name'],
                    last_name=profile['last_name'],
                    verified=profile['verified'],
                    name=profile['name'],
                    language=profile['locale'],
                    hometown=hometown,
                    email=profile['email'],
                    gender=profile['gender'],
                    dob=bday,
                    location=location,
                    timezone=profile['timezone'],
                    accesstoken=accesstoken
                )
            else:
                fb.user = user,
                fb.facebook_url = profile['link'],
                fb.facebook_id = profile['id'],
                fb.first_name = profile['first_name'],
                fb.last_name = profile['last_name'],
                fb.verified = profile['verified'],
                fb.name = profile['name'],
                fb.language = profile['locale'],
                fb.hometown = hometown,
                fb.email = profile['email'],
                fb.gender = profile['gender'],
                fb.dob = bday,
                fb.location = location,
                fb.timezone = profile['timezone'],
                fb.accesstoken = accesstoken
                fb.save()
            if not request.user.is_authenticated():
                if not hasattr(user, 'backend'):
                    for backend in settings.AUTHENTICATION_BACKENDS:
                        if user == load_backend(backend).get_user(user.pk):
                            user.backend = backend
                            break
                if hasattr(user, 'backend'):
                    auth.login(request, user)
            messages.success(request, "Loggedin successfully")
            return HttpResponseRedirect(reverse_lazy('blog'))
        else:
            message.error(request, "Sorry, We didnt find your email id through facebook")
            return render(request, '404.html')
    elif 'error' in request.GET:
        print(request.GET)
    else:
        rty = "https://graph.facebook.com/oauth/authorize?client_id=" + os.getenv("FB_APP_ID") + "&redirect_uri=" + 'https://' + request.META['HTTP_HOST'] + reverse('facebook_login') + "&scope=manage_pages,read_stream, user_about_me, user_birthday, user_location, user_work_history, user_hometown, user_website, email, user_likes, user_groups"
        return HttpResponseRedirect(rty)


class ChangePasswordView(LoginRequiredMixin, FormView):
    template_name = "dashboard/user/change_password.html"
    form_class = ChangePasswordForm
    success_url = reverse_lazy("blog")

    def form_valid(self, form):
        user = self.request.user
        user.set_password(form.cleaned_data.get("password"))
        user.save()
        user = authenticate(username=user.username, password=form.cleaned_data.get("password"))
        login(self.request, user)
        messages.success(self.request, "your password has been changed!!!")
        return JsonResponse({"error": False, "success_url": reverse("blog")})

    def form_invalid(self, form):
        return JsonResponse({"error": True, "errors": form.errors})



import datetime
import os
from django.db import models
from django.template.defaultfilters import slugify
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import send_mail


ROLE_CHOICE = (
    ('Admin', 'Admin'),
    ('Publisher', 'Publisher'),
    ('Author', 'Author'),
)

STATUS_CHOICE = (
    ('Drafted', 'Drafted'),
    ('Published', 'Published'),
    ('Rejected', 'Rejected'),
    ('Trashed', 'Trashed'),
)


class UserRole(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    role = models.CharField(max_length=10, choices=STATUS_CHOICE)
    class Meta:
        ordering = ['-id']

class Theme(models.Model):
    name = models.CharField(max_length=20, unique=True)
    slug = models.CharField(max_length=20, unique=True)
    description = models.CharField(max_length=500)
    enabled = models.BooleanField(default=False)

    def save(self, *args, **kwargs):
        self.slug = slugify(self.name)
        super(Theme, self).save(*args, **kwargs)

    def __str__(self):
        return self.name


class Category(models.Model):
    name = models.CharField(max_length=20, unique=True)
    slug = models.CharField(max_length=20, unique=True)
    description = models.CharField(max_length=500)
    is_active = models.BooleanField(default=False)
    meta_description = models.TextField(max_length=160, null=True, blank=True)
    meta_keywords = models.TextField(max_length=255, null=True, blank=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)

    class Meta:
        ordering = ['-id']

    def save(self, *args, **kwargs):
        self.slug = slugify(self.name)
        super(Category, self).save(*args, **kwargs)

    def __str__(self):
        return self.name

    def category_posts(self):
        return Post.objects.filter(category=self).count()


class Tags(models.Model):
    name = models.CharField(max_length=20, unique=True)
    slug = models.CharField(max_length=20, unique=True)

    def save(self, *args, **kwargs):
        tempslug = slugify(self.name)
        if self.id:
            tag = Tags.objects.get(pk=self.id)
            if tag.name != self.name:
                self.slug = create_tag_slug(tempslug)
        else:
            self.slug = create_tag_slug(tempslug)
        super(Tags, self).save(*args, **kwargs)

    def __str__(self):
        return self.name


def create_tag_slug(tempslug):
    slugcount = 0
    while True:
        try:
            Tags.objects.get(slug=tempslug)
            slugcount += 1
            tempslug = tempslug + '-' + str(slugcount)
        except ObjectDoesNotExist:
            return tempslug


class Post(models.Model):
    title = models.CharField(max_length=100, unique=True)
    slug = models.SlugField(max_length=100, unique=True)
    created_on = models.DateTimeField(auto_now_add=True)
    updated_on = models.DateField(auto_now=True)
    meta_description = models.TextField(max_length=160, null=True, blank=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    content = models.TextField()
    category = models.ForeignKey(Category, on_delete=models.CASCADE)
    tags = models.ManyToManyField(Tags, related_name='rel_posts')
    status = models.CharField(max_length=10, choices=STATUS_CHOICE, default='Drafted')
    keywords = models.TextField(max_length=500, blank=True)
    featured_image = models.ImageField(upload_to='static/blog/uploads/%Y/%m/%d/', blank=True, null=True)

    class Meta:
        ordering = ['-updated_on']

    def save(self, *args, **kwargs):
        tempslug = slugify(self.title)
        if self.id:
            blogpost = Post.objects.get(pk=self.id)
            if blogpost.title != self.title:
                self.slug = create_slug(tempslug)
        else:
            self.slug = create_slug(tempslug)
            self.email_to_admins_on_post_create()

        super(Post, self).save(*args, **kwargs)

    def __str__(self):
        return self.title

    def is_deletable_by(self, user):
        if self.user == user or user.is_superuser:
            return True
        return False

    def create_activity(self, user, content):
        return PostHistory.objects.create(
            user=user, post=self, content=content
        )

    def create_activity_instance(self, user, content):
        return PostHistory(
            user=user, post=self, content=content
        )

    def remove_activity(self):
        self.history.all().delete()

    def email_to_admins_on_post_create(self):
        email = os.getenv("DEFAULT_EMAIL")
        if not self.id and email:
            admin_roles = UserRole.objects.select_related().filter(role="Admin")
            admin_emails = [admin_role.user.email for admin_role in admin_roles]
            user = self.user
            author_name = user.first_name + user.last_name if user.first_name else user.email
            text = "New blog post has been created by {0} with the name {1} in the category {2}.".format(author_name, self.title, self.category.name)
            send_mail(
                subject="New Blog Post created",
                message=text,
                from_email=email,
                recipient_list=admin_emails,
                fail_silently=False,
            )

    def store_old_slug(self, old_slug):
        query = Post_Slugs.objects.filter(blog=self, slug=old_slug).values_list("slug", flat=True)
        if not (query and old_slug != self.slug):
            old_slug, _ = Post_Slugs.objects.get_or_create(blog=self, slug=old_slug)
            old_slug.is_active = False
            old_slug.save()
        active_slug, _ = Post_Slugs.objects.get_or_create(blog=self, slug=self.slug)
        active_slug.is_active = True
        active_slug.save()


def create_slug(tempslug):
    slugcount = 0
    while True:
        try:
            Post.objects.get(slug=tempslug)
            slugcount += 1
            tempslug = tempslug + '-' + str(slugcount)
        except ObjectDoesNotExist:
            return tempslug


class Post_Slugs(models.Model):
    blog = models.ForeignKey(Post, related_name='slugs', on_delete=models.CASCADE)
    slug = models.SlugField(max_length=100, unique=True)
    is_active = models.BooleanField(default=False)

    def __str__(self):
        return self.slug


class PostHistory(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    post = models.ForeignKey(Post, related_name='history', on_delete=models.CASCADE)
    content = models.CharField(max_length=200)
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return '{username} {content} {blog_title}'.format(
            username=str(self.user.get_username()),
            content=str(self.content),
            blog_title=str(self.post.title)
        )


class Image_File(models.Model):
    upload = models.FileField(upload_to="static/uploads/%Y/%m/%d/")
    date_created = models.DateTimeField(default=datetime.datetime.now)
    is_image = models.BooleanField(default=True)
    thumbnail = models.FileField(upload_to="static/uploads/%Y/%m/%d/", blank=True, null=True)

    def __str__(self):
        return self.date_created


class Page(models.Model):
    title = models.CharField(max_length=100)
    content = models.TextField()
    slug = models.SlugField()
    is_active = models.BooleanField(default=True)
    meta_description = models.TextField()
    keywords = models.TextField()
    meta_title = models.TextField()

    class Meta:
        ordering = ['-id']

    def save(self, *args, **kwargs):
        tempslug = slugify(self.title)
        if self.id:
            existed_page = Page.objects.get(pk=self.id)
            if existed_page.title != self.title:
                self.slug = create_slug(tempslug)
        else:
            self.slug = create_slug(tempslug)

        super(Page, self).save(*args, **kwargs)

    def __str__(self):
        return self.title


class Menu(models.Model):
    parent = models.ForeignKey('self', blank=True, null=True, on_delete=models.CASCADE)
    title = models.CharField(max_length=255)
    url = models.URLField(max_length=255, blank=True)
    status = models.BooleanField(default=True)
    lvl = models.IntegerField(blank=True)

    def __str__(self):
        return self.title

    def get_children(self):
        return self.menu_set.all()

    def has_children(self):
        if self.get_children():
            return True
        return False


class ContactUsSettings(models.Model):
    from_email = models.EmailField()
    reply_to_email = models.EmailField(blank=True, null=True)
    email_admin = models.EmailField()
    subject = models.CharField(max_length=500)
    body_user = models.TextField()
    body_admin = models.TextField()


class Google(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='google', on_delete=models.CASCADE)
    google_id = models.CharField(max_length=200, default='')
    google_url = models.CharField(max_length=1000, default='')
    verified_email = models.CharField(max_length=200, default='')
    family_name = models.CharField(max_length=200, default='')
    name = models.CharField(max_length=200, default='')
    picture = models.CharField(max_length=200, default='')
    gender = models.CharField(max_length=10, default='')
    dob = models.CharField(max_length=50, default='')
    given_name = models.CharField(max_length=200, default='')
    email = models.CharField(max_length=200, default='', db_index=True)

    def __str__(self):
        return self.email


class Facebook(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='facebook', on_delete=models.CASCADE)
    facebook_id = models.CharField(max_length=100)
    facebook_url = models.CharField(max_length=200, default='')
    first_name = models.CharField(max_length=200, default='')
    last_name = models.CharField(max_length=200, default='')
    verified = models.CharField(max_length=200, default='')
    name = models.CharField(max_length=200, default='')
    language = models.CharField(max_length=200, default='')
    hometown = models.CharField(max_length=200, default='')
    email = models.CharField(max_length=200, default='', db_index=True)
    gender = models.CharField(max_length=200, default='')
    dob = models.DateField(null=True, blank=True)
    location = models.CharField(max_length=200, default='')
    timezone = models.CharField(max_length=200, default='')
    accesstoken = models.CharField(max_length=2000, default='')

    def __str__(self):
        return self.email



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_blog_it', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='post',
            name='tags',
        ),
        migrations.AddField(
            model_name='post',
            name='tags',
            field=models.ManyToManyField(related_name='rel_posts', to='django_blog_it.Tags'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-19 11:40
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_blog_it', '0010_theme'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='meta_description',
            field=models.TextField(blank=True, max_length=160, null=True),
        ),
        migrations.AddField(
            model_name='category',
            name='meta_keywords',
            field=models.TextField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='post',
            name='meta_description',
            field=models.TextField(blank=True, max_length=160, null=True),
        ),
        migrations.AlterField(
            model_name='post',
            name='slug',
            field=models.SlugField(max_length=100, unique=True),
        ),
        migrations.AlterField(
            model_name='post',
            name='title',
            field=models.CharField(max_length=100, unique=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-01 08:34
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_blog_it', '0008_post_featured_image'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContactUsSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('from_email', models.EmailField(max_length=254)),
                ('reply_to_email', models.EmailField(blank=True, max_length=254, null=True)),
                ('email_admin', models.EmailField(max_length=254)),
                ('subject', models.CharField(max_length=500)),
                ('body_user', models.TextField()),
                ('body_admin', models.TextField()),
            ],
        ),
    ]



import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))
PROJECT_NAME = 'django_blog_it'

data_files = []
for dirpath, dirnames, filenames in os.walk(PROJECT_NAME):
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'):
            del dirnames[i]
    if '__init__.py' in filenames:
        continue
    elif filenames:
        for f in filenames:
            data_files.append(os.path.join(
                dirpath[len(PROJECT_NAME) + 1:], f))

setup(
    name='django-blog-it',
    version='0.3.1',
    packages=find_packages(exclude=['tests', 'tests.*']),
    include_package_data=True,
    description='A simple installable app for writing blog posts',
    long_description=README,
    url='https://github.com/MicroPyramid/django-blog-it.git',
    author='Micropyramid',
    author_email='hello@micropyramid.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'boto',
        'Django',
        'django-simple-pagination',
        'django-storages',
        'Pillow',
        'requests',
        'wheel',
        'microurl'
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_blog_it', '0002_auto_20160223_0521'),
    ]

    operations = [
        migrations.AlterField(
            model_name='post',
            name='status',
            field=models.CharField(default=b'Drafted', max_length=10, choices=[(b'Drafted', b'Drafted'), (b'Published', b'Published'), (b'Rejected', b'Rejected'), (b'Trashed', b'Trashed')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-18 10:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_blog_it', '0009_contactussettings'),
    ]

    operations = [
        migrations.CreateModel(
            name='Theme',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=20, unique=True)),
                ('slug', models.CharField(max_length=20, unique=True)),
                ('description', models.CharField(max_length=500)),
                ('enabled', models.BooleanField(default=False)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-22 06:57
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_blog_it', '0011_auto_20160719_1140'),
    ]

    operations = [
        migrations.CreateModel(
            name='Google',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('google_id', models.CharField(default='', max_length=200)),
                ('google_url', models.CharField(default='', max_length=1000)),
                ('verified_email', models.CharField(default='', max_length=200)),
                ('family_name', models.CharField(default='', max_length=200)),
                ('name', models.CharField(default='', max_length=200)),
                ('picture', models.CharField(default='', max_length=200)),
                ('gender', models.CharField(default='', max_length=10)),
                ('dob', models.CharField(default='', max_length=50)),
                ('given_name', models.CharField(default='', max_length=200)),
                ('email', models.CharField(db_index=True, default='', max_length=200)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='google', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterField(
            model_name='post',
            name='featured_image',
            field=models.ImageField(blank=True, null=True, upload_to='static/blog/uploads/%Y/%m/%d/'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-28 11:37
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_blog_it', '0013_facebook'),
    ]

    operations = [
        migrations.CreateModel(
            name='Post_Slugs',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(max_length=100, unique=True)),
                ('is_active', models.BooleanField(default=False)),
                ('blog', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='slugs', to='django_blog_it.Post')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-31 11:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_blog_it', '0007_menu'),
    ]

    operations = [
        migrations.AddField(
            model_name='post',
            name='featured_image',
            field=models.ImageField(blank=True, null=True, upload_to=b'static/blog/uploads/%Y/%m/%d/'),
        ),
    ]



import os
import requests
import json
from datetime import datetime
from django.shortcuts import render, get_list_or_404, get_object_or_404
from django_blog_it.django_blog_it.models import Post, Category, Tags, Page
from django_blog_it.django_blog_it.forms import ContactForm
from django.db.models import Count
from django_blog_it import settings
from django.contrib import messages
from django.http import JsonResponse, HttpResponseRedirect
from django.core.mail import EmailMultiAlternatives
from django.template import Context
from django.views.generic import ListView, DetailView
from django.shortcuts import render_to_response
from django.urls import reverse
from microurl import google_mini
from django_blog_it.django_blog_it.models import ContactUsSettings, Post_Slugs


def categories_tags_lists():
    categories_list = Category.objects.filter(is_active=True, post__status='Published').distinct()
    tags_list = Tags.objects.annotate(
        Num=Count('rel_posts')).filter(Num__gt=0, rel_posts__status='Published', rel_posts__category__is_active=True)[:20]
    posts = Post.objects.filter(status='Published').order_by('-updated_on')[0:3]
    return {'categories_list': categories_list, 'tags_list': tags_list, 'recent_posts': posts}


# def seperate_tags():
#     posts_tags = Post.objects.filter(category__is_active=True, status='Published')
#     for blog in posts_tags:
#         blog_tags_new = blog.tags.split(',')
#         for tag in blog_tags_new:
#             real_tags = Tags.objects.get(slug=tag)
#             return real_tags


class Home(ListView):
    template_name = "posts/new_index.html"
    queryset = Post.objects.filter(status='Published', category__is_active=True).order_by('-updated_on')
    context_object_name = "blog_posts"

    def get_context_data(self, *args, **kwargs):
        context = super(Home, self).get_context_data(*args, **kwargs)
        context.update({
            "description": settings.BLOG_DESCRIPTION,
            "title": settings.BLOG_TITLE,
            "keywords": settings.BLOG_KEYWORDS,
            "author": settings.BLOG_AUTHOR,
        })
        context.update(categories_tags_lists())
        return context


class BlogPostView(DetailView):
    template_name = 'posts/new_blog_view.html'
    model = Post
    slug_url_kwarg = "blog_slug"
    context_object_name = "blog_name"

    def dispatch(self, request, *args, **kwargs):
        self.object = Post.objects.filter(slug=kwargs.get("blog_slug")).last()
        if not self.object:
            post_slug = get_object_or_404(Post_Slugs, slug=self.kwargs.get("blog_slug"))
            if self.kwargs.get("blog_slug") != post_slug.blog.slug:
                return HttpResponseRedirect(reverse('blog_post_view', kwargs={"blog_slug": post_slug.blog.slug}), status=301)
        return super(BlogPostView, self).dispatch(request, *args, **kwargs)

    def get_mini_url(self, request):
        url = request.build_absolute_uri()
        try:
            api_key = os.getenv('API_KEY')
            url = google_mini(url, api_key)
        except Exception:
            pass
        return url

    def get_context_data(self, *args, **kwargs):
        context = super(BlogPostView, self).get_context_data(*args, **kwargs)
        user = self.object.user
        author = user.first_name if user.first_name else user.username
        related_posts = Post.objects.filter(
            status='Published',
            category=self.object.category,
            tags__in=self.object.tags.all()
        ).exclude(id=self.object.id).distinct()[:3]
        context.update({
            "related_posts": related_posts,
            "disqus_shortname": getattr(settings, 'DISQUS_SHORTNAME'),
            "description": self.object.meta_description if self.object.meta_description else "",
            "title": self.object.title,
            "keywords": self.object.keywords,
            "author": author,
            "short_url": self.get_mini_url(self.request),
            "blog_title": settings.BLOG_TITLE
        })
        context.update(categories_tags_lists())
        return context


class SelectedCategoryView(ListView):
    template_name = "posts/new_index.html"
    context_object_name = "blog_posts"

    def get_queryset(self):
        self.category = get_object_or_404(Category, slug=self.kwargs.get("category_slug"))
        return Post.objects.filter(category=self.category, category__is_active=True, status='Published')

    def get_context_data(self, *args, **kwargs):
        context = super(SelectedCategoryView, self).get_context_data(*args, **kwargs)
        user = self.category.user
        author = user.first_name if user.first_name else user.username
        context.update({
            "description": self.category.description,
            "title": self.category.name,
            "keywords": self.category.meta_keywords,
            "author": author,
            "category": self.category,
        })
        context.update(categories_tags_lists())
        return context


class SelectedTagView(ListView):
    template_name = "posts/new_index.html"
    context_object_name = "blog_posts"

    def get_queryset(self):
        self.tag = get_object_or_404(Tags, slug=self.kwargs.get("tag_slug"))
        return get_list_or_404(Post, tags=self.tag, status='Published', category__is_active=True)

    def get_context_data(self, *args, **kwargs):
        context = super(SelectedTagView, self).get_context_data(*args, **kwargs)
        context.update({
            "description": self.tag.name,
            "title": self.tag.name,
            "keywords": self.tag.name,
            "author": settings.BLOG_AUTHOR,
            "tag": self.tag,
        })
        context.update(categories_tags_lists())
        return context


class ArchiveView(ListView):
    template_name = "posts/new_index.html"
    context_object_name = "blog_posts"

    def get_queryset(self):
        year = self.kwargs.get("year")
        month = self.kwargs.get("month")
        self.date = datetime(int(year), int(month), 1)
        return Post.objects.filter(
            category__is_active=True, status="Published", updated_on__year=year, updated_on__month=month).order_by('-updated_on')

    def get_context_data(self, *args, **kwargs):
        context = super(ArchiveView, self).get_context_data(*args, **kwargs)
        context.update({
            "description": "Blog Archive - " + self.date.strftime("%B %Y"),
            "title": "Blog Archive - " + self.date.strftime("%B %Y"),
            "keywords": "Blog Archive - " + self.date.strftime("%B %Y"),
            "author": settings.BLOG_AUTHOR,
            "date": self.date,
        })
        context.update(categories_tags_lists())
        return context


class PageView(DetailView):
    template_name = "posts/page.html"
    model = Page
    slug_url_kwarg = "page_slug"
    context_object_name = "page"


def contact_us(request):
    form = ContactForm()
    if request.POST:
        form = ContactForm(request.POST)
        if form.is_valid():
            if os.getenv("GOOGLE_CAPTCHA_SECRET_KEY"):
                payload = {'secret': os.getenv("GOOGLE_CAPTCHA_SECRET_KEY"),
                           'response': request.POST.get('g-recaptcha-response'),
                           'remoteip': request.META.get('REMOTE_ADDR')}
                r = requests.get('https://www.google.com/recaptcha/api/siteverify', params=payload)
                if not json.loads(r.text)['success']:
                    return JsonResponse({'error': True, 'response': {"captcha": "Invalid captcha"}})
            # email sending
            contact_us = ContactUsSettings.objects.last()
            # email to admin
            subject = 'Blog Suggestions - ' + form.cleaned_data.get("contact_name")
            from_email = form.cleaned_data.get("contact_email")
            context = Context({
                "NAME_OF_USER": form.cleaned_data.get("contact_name"),
                "WEBSITE_OF_USER": form.cleaned_data.get("contact_website"),
                "USER_DESCRIPTION": form.cleaned_data.get("content"),
                "BLOG_TITLE": settings.BLOG_TITLE
            })
            html_content = render_to_response('emails/email_to_admin.html', context).content.decode("utf-8")
            msg = EmailMultiAlternatives(subject, subject, from_email, [contact_us.email_admin])
            # msg.attach(html_content, 'text/html')
            msg.attach_alternative(html_content, "text/html")
            msg.send()
            # email to user
            subject = 'Thank you for contacting us - ' + settings.BLOG_TITLE
            from_email = contact_us.from_email
            context = Context({
                "BODY_USER": contact_us.body_user,
                "BLOG_TITLE": settings.BLOG_TITLE
            })
            html_content = render_to_response('emails/email_to_user.html', context).content.decode("utf-8")
            headers = {'Reply-To': contact_us.reply_to_email}
            msg = EmailMultiAlternatives(subject, subject, from_email, [form.cleaned_data.get("contact_email")], headers=headers)
            msg.attach_alternative(html_content, "text/html")
            msg.send()
            # end
            messages.success(
                request, 'Successfully Sent your contact us details.')
            data = {'error': False,
                    'response': 'Successfully Sent your contact us details.'}
        else:
            data = {'error': True, 'response': form.errors}
        return JsonResponse(data)

    context = {"description": settings.BLOG_DESCRIPTION,
               "title": settings.BLOG_TITLE,
               "keywords": settings.BLOG_KEYWORDS,
               "author": settings.BLOG_AUTHOR,
               "contact_form": form}

    if os.getenv("GOOGLE_CAPTCHA_SITE_KEY"):
        context.update({"GOOGLE_CAPTCHA_SITE_KEY": os.getenv("GOOGLE_CAPTCHA_SITE_KEY")})
    context.update(categories_tags_lists())
    return render(request, 'posts/contact.html', context)



# Generated by Django 2.0.2 on 2018-03-04 10:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_blog_it', '0014_post_slugs'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='category',
            options={'ordering': ['-id']},
        ),
        migrations.AlterModelOptions(
            name='page',
            options={'ordering': ['-id']},
        ),
        migrations.AlterModelOptions(
            name='post',
            options={'ordering': ['-updated_on']},
        ),
        migrations.AlterModelOptions(
            name='userrole',
            options={'ordering': ['-id']},
        ),
    ]
